@@ -2,16 +2,17 @@
 #include <stdlib.h>
 #include "lib/mlrutil.h"
 #include "containers/percentile_keeper.h"
+#include "containers/mvfuncs.h"
 
 #define INITIAL_CAPACITY 10000
 #define GROWTH_FACTOR    2.0
 
 // ----------------------------------------------------------------
 percentile_keeper_t* percentile_keeper_alloc() {
-	int capacity = INITIAL_CAPACITY;
+	unsigned long long capacity = INITIAL_CAPACITY;
 	percentile_keeper_t* ppercentile_keeper = mlr_malloc_or_die(sizeof(percentile_keeper_t));
 	ppercentile_keeper->data     = mlr_malloc_or_die(capacity*sizeof(mv_t));
-	ppercentile_keeper->size     = 0;
+	ppercentile_keeper->size     = 0LL;
 	ppercentile_keeper->capacity = capacity;
 	ppercentile_keeper->sorted   = FALSE;
 	return ppercentile_keeper;
@@ -21,17 +22,20 @@ percentile_keeper_t* percentile_keeper_alloc() {
 void percentile_keeper_free(percentile_keeper_t* ppercentile_keeper) {
 	if (ppercentile_keeper == NULL)
 		return;
+	for (unsigned long long i = 0; i < ppercentile_keeper->size; i++) {
+		mv_free(&ppercentile_keeper->data[i]);
+	}
 	free(ppercentile_keeper->data);
 	ppercentile_keeper->data = NULL;
-	ppercentile_keeper->size = 0;
-	ppercentile_keeper->capacity = 0;
+	ppercentile_keeper->size = 0LL;
+	ppercentile_keeper->capacity = 0LL;
 	free(ppercentile_keeper);
 }
 
 // ----------------------------------------------------------------
 void percentile_keeper_ingest(percentile_keeper_t* ppercentile_keeper, mv_t value) {
 	if (ppercentile_keeper->size >= ppercentile_keeper->capacity) {
-		ppercentile_keeper->capacity = (int)(ppercentile_keeper->capacity * GROWTH_FACTOR);
+		ppercentile_keeper->capacity = (unsigned long long)(ppercentile_keeper->capacity * GROWTH_FACTOR);
 		ppercentile_keeper->data = (mv_t*)mlr_realloc_or_die(ppercentile_keeper->data,
 			ppercentile_keeper->capacity*sizeof(mv_t));
 	}
@@ -209,23 +213,23 @@ void percentile_keeper_ingest(percentile_keeper_t* ppercentile_keeper, mv_t valu
 // * (Note that Miller's interpolated percentiles match match R's quantile with type=7)
 // ----------------------------------------------------------------
 
-static int compute_index_non_interpolated(int n, double p) {
-	int index = p*n/100.0;
-	//int index = p*(n-1)/100.0;
-	//int index = (int)ceil(p*(n-1)/100.0);
-	//int index = (int)ceil(-0.5 + p*(n-1)/100.0);
+static unsigned long long compute_index_non_interpolated(unsigned long long n, double p) {
+	long long index = p*n/100.0;
+	//unsigned long long index = p*(n-1)/100.0;
+	//unsigned long long index = (unsigned long long)ceil(p*(n-1)/100.0);
+	//unsigned long long index = (unsigned long long)ceil(-0.5 + p*(n-1)/100.0);
 	if (index >= n)
 		index = n-1;
 	if (index < 0)
 		index = 0;
-	return index;
+	return (unsigned long long)index;
 }
 
-static mv_t get_percentile_linearly_interpolated(mv_t* array, int n, double p) {
+static mv_t get_percentile_linearly_interpolated(mv_t* array, unsigned long long n, double p) {
 	double findex = (p/100.0)*(n-1);
 	if (findex < 0)
 		findex = 0;
-	int iindex = (int)floor(findex);
+	unsigned long long iindex = (unsigned long long)floor(findex);
 	if (iindex >= n-1) {
 		return array[iindex];
 	} else {
@@ -246,7 +250,7 @@ mv_t percentile_keeper_emit_non_interpolated(percentile_keeper_t* ppercentile_ke
 		return mv_absent();
 	}
 	if (!ppercentile_keeper->sorted) {
-		qsort(ppercentile_keeper->data, ppercentile_keeper->size, sizeof(mv_t), mv_nn_comparator);
+		qsort(ppercentile_keeper->data, ppercentile_keeper->size, sizeof(mv_t), mv_xx_comparator);
 		ppercentile_keeper->sorted = TRUE;
 	}
 	return ppercentile_keeper->data[compute_index_non_interpolated(ppercentile_keeper->size, percentile)];
@@ -257,7 +261,7 @@ mv_t percentile_keeper_emit_linearly_interpolated(percentile_keeper_t* ppercenti
 		return mv_absent();
 	}
 	if (!ppercentile_keeper->sorted) {
-		qsort(ppercentile_keeper->data, ppercentile_keeper->size, sizeof(mv_t), mv_nn_comparator);
+		qsort(ppercentile_keeper->data, ppercentile_keeper->size, sizeof(mv_t), mv_xx_comparator);
 		ppercentile_keeper->sorted = TRUE;
 	}
 	return get_percentile_linearly_interpolated(ppercentile_keeper->data, ppercentile_keeper->size, percentile);
@@ -266,11 +270,11 @@ mv_t percentile_keeper_emit_linearly_interpolated(percentile_keeper_t* ppercenti
 // ----------------------------------------------------------------
 void percentile_keeper_print(percentile_keeper_t* ppercentile_keeper) {
 	printf("percentile_keeper dump:\n");
-	for (int i = 0; i < ppercentile_keeper->size; i++) {
+	for (unsigned long long i = 0; i < ppercentile_keeper->size; i++) {
 		mv_t* pa = &ppercentile_keeper->data[i];
 		if (pa->type == MT_FLOAT)
-			printf("[%02d] %.8lf\n", i, ppercentile_keeper->data[i].u.fltv);
+			printf("[%02llu] %.8lf\n", i, ppercentile_keeper->data[i].u.fltv);
 		else
-			printf("[%02d] %8lld\n", i, ppercentile_keeper->data[i].u.intv);
+			printf("[%02llu] %8lld\n", i, ppercentile_keeper->data[i].u.intv);
 	}
 }
