@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -26,11 +35,12 @@
 
 // AKA "token"
 #define IRS_STRIDX           0x2001
-#define IFS_STRIDX           0x2003
-#define DQUOTE_STRIDX        0x2004
-#define DQUOTE_IRS_STRIDX    0x2005
+#define IFS_STRIDX           0x2002
+#define DQUOTE_STRIDX        0x2003
+#define DQUOTE_IRS_STRIDX    0x2004
+#define DQUOTE_IRS2_STRIDX   0x2005 // alternate line-ending for autodetect LF/CRLF
 #define DQUOTE_IFS_STRIDX    0x2006
-#define DQUOTE_DQUOTE_STRIDX 0x2008
+#define DQUOTE_DQUOTE_STRIDX 0x2007
 
 // ----------------------------------------------------------------
 typedef struct _lrec_reader_mmap_csv_state_t {
@@ -44,9 +54,11 @@ typedef struct _lrec_reader_mmap_csv_state_t {
 	char* ifs;
 	char* dquote;
 	char* dquote_irs;
+	char* dquote_irs2;
 	char* dquote_ifs;
 	char* dquote_eof;
 	char* dquote_dquote;
+	int   do_auto_line_term;
 
 	int   dquotelen;
 
@@ -67,7 +79,7 @@ static void    lrec_reader_mmap_csv_free(lrec_reader_t* preader);
 static void    lrec_reader_mmap_csv_sof(void* pvstate, void* pvhandle);
 static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, context_t* pctx);
 static int     lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
-	rslls_t* pfields, file_reader_mmap_state_t* phandle);
+	rslls_t* pfields, file_reader_mmap_state_t* phandle, context_t* pctx);
 static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx);
 static lrec_t* paste_header_and_data(lrec_reader_mmap_csv_state_t* pstate, rslls_t* pdata_fields, context_t* pctx);
 
@@ -78,13 +90,18 @@ lrec_reader_t* lrec_reader_mmap_csv_alloc(char* irs, char* ifs, int use_implicit
 	lrec_reader_mmap_csv_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_mmap_csv_state_t));
 	pstate->ilno          = 0LL;
 
+	pstate->do_auto_line_term = FALSE;
+	if (streq(irs, "auto")) {
+		pstate->do_auto_line_term = TRUE;
+		irs = "\n";
+	}
+
 	pstate->eof           = "\xff";
 	pstate->irs           = irs;
 	pstate->ifs           = ifs;
 	pstate->ifs_eof       = mlr_paste_2_strings(pstate->ifs, "\xff");
 	pstate->dquote        = "\"";
 
-	pstate->dquote_irs    = mlr_paste_2_strings("\"", pstate->irs);
 	pstate->dquote_ifs    = mlr_paste_2_strings("\"", pstate->ifs);
 	pstate->dquote_eof    = "\"\xff";
 	pstate->dquote_dquote = "\"\"";
@@ -97,7 +114,16 @@ lrec_reader_t* lrec_reader_mmap_csv_alloc(char* irs, char* ifs, int use_implicit
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_STRIDX);
 
 	pstate->pdquote_parse_trie = parse_trie_alloc();
-	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,    DQUOTE_IRS_STRIDX);
+	if (pstate->do_auto_line_term) {
+		pstate->dquote_irs  = mlr_paste_2_strings("\"", "\n");
+		pstate->dquote_irs2 = mlr_paste_2_strings("\"", "\r\n");
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_STRIDX);
+	} else {
+		pstate->dquote_irs  = mlr_paste_2_strings("\"", pstate->irs);
+		pstate->dquote_irs2 = NULL;
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_STRIDX);
+	}
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_STRIDX);
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_dquote, DQUOTE_DQUOTE_STRIDX);
 
@@ -133,6 +159,7 @@ static void lrec_reader_mmap_csv_free(lrec_reader_t* preader) {
 	sb_free(pstate->psb);
 	free(pstate->ifs_eof);
 	free(pstate->dquote_irs);
+	free(pstate->dquote_irs2);
 	free(pstate->dquote_ifs);
 	free(pstate);
 	free(preader);
@@ -151,7 +178,7 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 	file_reader_mmap_state_t* phandle = pvhandle;
 
 	if (pstate->expect_header_line_next) {
-		if (!lrec_reader_mmap_csv_get_fields(pstate, pstate->pfields, phandle))
+		if (!lrec_reader_mmap_csv_get_fields(pstate, pstate->pfields, phandle, pctx))
 			return NULL;
 		pstate->ilno++;
 
@@ -165,7 +192,12 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 			}
 			// Transfer pointer-free responsibility from the rslls to the
 			// header fields in the header keeper
-			slls_append(pheader_fields, pe->value, pe->free_flag);
+			if (string_starts_with(pe->value, "\xef\xbb\xbf")) {
+				// Strip UTF-8 BOM if any
+				slls_append(pheader_fields, mlr_strdup_or_die(&pe->value[3]), FREE_ENTRY_VALUE);
+			} else {
+				slls_append(pheader_fields, pe->value, pe->free_flag);
+			}
 			pe->free_flag = 0;
 		}
 		rslls_reset(pstate->pfields);
@@ -181,7 +213,7 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 
 		pstate->expect_header_line_next = FALSE;
 	}
-	int rc = lrec_reader_mmap_csv_get_fields(pstate, pstate->pfields, phandle);
+	int rc = lrec_reader_mmap_csv_get_fields(pstate, pstate->pfields, phandle, pctx);
 	pstate->ilno++;
 	if (rc == FALSE) // EOF
 		return NULL;
@@ -195,7 +227,7 @@ static lrec_t* lrec_reader_mmap_csv_process(void* pvstate, void* pvhandle, conte
 }
 
 static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
-	rslls_t* pfields, file_reader_mmap_state_t* phandle)
+	rslls_t* pfields, file_reader_mmap_state_t* phandle, context_t* pctx)
 {
 	int rc, stridx, matchlen, record_done, field_done;
 	string_builder_t* psb = pstate->psb;
@@ -215,11 +247,7 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 			// Loop over characters in field
 			field_done = FALSE;
 			while (!field_done) {
-				if (e > phandle->eof) {
-					fprintf(stderr, "%s: internal coding error detected at line %d of file %s.\n",
-						MLR_GLOBALS.bargv0, __LINE__, __FILE__);
-					exit(1);
-				}
+				MLR_INTERNAL_CODING_ERROR_IF(e > phandle->eof);
 				rc = parse_trie_match(pstate->pno_dquote_parse_trie, e, phandle->eof, &stridx, &matchlen);
 				if (rc) {
 					switch(stridx) {
@@ -231,6 +259,16 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						break;
 					case IRS_STRIDX: // end of record
 						*e = 0;
+
+						if (pstate->do_auto_line_term) {
+							if (e > p && e[-1] == '\r') {
+								e[-1] = 0;
+								context_set_autodetected_crlf(pctx);
+							} else {
+								context_set_autodetected_lf(pctx);
+							}
+						}
+
 						rslls_append(pfields, p, NO_FREE, 0);
 						p = e + matchlen;
 						field_done  = TRUE;
@@ -298,7 +336,18 @@ static int lrec_reader_mmap_csv_get_fields(lrec_reader_mmap_csv_state_t* pstate,
 						field_done  = TRUE;
 						break;
 					case DQUOTE_IRS_STRIDX: // end of record
+					case DQUOTE_IRS2_STRIDX: // end of record
 						*e = 0;
+
+						if (pstate->do_auto_line_term) {
+							if (e > p && e[-1] == '\r') {
+								e[-1] = 0;
+								context_set_autodetected_crlf(pctx);
+							} else {
+								context_set_autodetected_lf(pctx);
+							}
+						}
+
 						if (contiguous)
 							rslls_append(pfields, p, NO_FREE, FIELD_QUOTED_ON_INPUT);
 						else
@@ -342,7 +391,7 @@ static lrec_t* paste_indices_and_data(lrec_reader_mmap_csv_state_t* pstate, rsll
 	for (rsllse_t* pd = pdata_fields->phead; idx < pdata_fields->length && pd != NULL; pd = pd->pnext) {
 		idx++;
 		char free_flags = pd->free_flag;
-		char* key = make_nidx_key(idx, &free_flags);
+		char* key = low_int_to_string(idx, &free_flags);
 		// Transfer pointer-free responsibility from the rslls to the lrec object
 		lrec_put_ext(prec, key, pd->value, free_flags, pd->quote_flag);
 		pd->free_flag = 0;
