@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "lib/mlr_globals.h"
@@ -35,6 +44,7 @@ typedef struct _lrec_reader_mmap_csvlite_state_t {
 	int   irslen;
 	int   ifslen;
 	int   allow_repeat_ifs;
+	int   do_auto_line_term;
 	int   use_implicit_header;
 
 	int  expect_header_line_next;
@@ -48,7 +58,7 @@ static lrec_t* lrec_reader_mmap_csvlite_process_single_seps(void* pvstate, void*
 static lrec_t* lrec_reader_mmap_csvlite_process_multi_seps(void* pvstate, void* pvhandle, context_t* pctx);
 
 static slls_t* lrec_reader_mmap_csvlite_get_header_single_seps(file_reader_mmap_state_t* phandle,
-	lrec_reader_mmap_csvlite_state_t* pstate);
+	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx);
 
 static slls_t* lrec_reader_mmap_csvlite_get_header_multi_seps(file_reader_mmap_state_t* phandle,
 	lrec_reader_mmap_csvlite_state_t* pstate);
@@ -77,6 +87,7 @@ lrec_reader_t* lrec_reader_mmap_csvlite_alloc(char* irs, char* ifs, int allow_re
 	pstate->irslen                   = strlen(irs);
 	pstate->ifslen                   = strlen(ifs);
 	pstate->allow_repeat_ifs         = allow_repeat_ifs;
+	pstate->do_auto_line_term              = FALSE;
 	pstate->use_implicit_header      = use_implicit_header;
 	pstate->expect_header_line_next  = use_implicit_header ? FALSE : TRUE;
 	pstate->pheader_keeper           = NULL;
@@ -86,9 +97,22 @@ lrec_reader_t* lrec_reader_mmap_csvlite_alloc(char* irs, char* ifs, int allow_re
 	plrec_reader->popen_func    = file_reader_mmap_vopen;
 	plrec_reader->pclose_func   = file_reader_mmap_vclose;
 
-	plrec_reader->pprocess_func = (pstate->irslen == 1 && pstate->ifslen == 1)
-		? lrec_reader_mmap_csvlite_process_single_seps
-		: lrec_reader_mmap_csvlite_process_multi_seps;
+	if (streq(irs, "auto")) {
+		// Auto means either lines end in "\n" or "\r\n" (LF or CRLF).  In
+		// either case the final character is "\n". Then for autodetect we
+		// simply check if there's a character in the line before the '\n', and
+		// if that is '\r'.
+		pstate->do_auto_line_term = TRUE;
+		pstate->irs = "\n";
+		pstate->irslen = 1;
+		plrec_reader->pprocess_func = (pstate->ifslen == 1)
+			? lrec_reader_mmap_csvlite_process_single_seps
+			: lrec_reader_mmap_csvlite_process_multi_seps;
+	} else {
+		plrec_reader->pprocess_func = (pstate->irslen == 1 && pstate->ifslen == 1)
+			? lrec_reader_mmap_csvlite_process_single_seps
+			: lrec_reader_mmap_csvlite_process_multi_seps;
+	}
 
 	plrec_reader->psof_func     = lrec_reader_mmap_csvlite_sof;
 	plrec_reader->pfree_func    = lrec_reader_mmap_csvlite_free;
@@ -123,7 +147,7 @@ static lrec_t* lrec_reader_mmap_csvlite_process_single_seps(void* pvstate, void*
 	while (TRUE) {
 		if (pstate->expect_header_line_next) {
 
-			slls_t* pheader_fields = lrec_reader_mmap_csvlite_get_header_single_seps(phandle, pstate);
+			slls_t* pheader_fields = lrec_reader_mmap_csvlite_get_header_single_seps(phandle, pstate, pctx);
 			if (pheader_fields == NULL) { // EOF
 				return NULL;
 			}
@@ -211,7 +235,7 @@ static lrec_t* lrec_reader_mmap_csvlite_process_multi_seps(void* pvstate, void*
 
 // ----------------------------------------------------------------
 static slls_t* lrec_reader_mmap_csvlite_get_header_single_seps(file_reader_mmap_state_t* phandle,
-	lrec_reader_mmap_csvlite_state_t* pstate)
+	lrec_reader_mmap_csvlite_state_t* pstate, context_t* pctx)
 {
 	char irs = pstate->irs[0];
 	char ifs = pstate->ifs[0];
@@ -235,6 +259,16 @@ static slls_t* lrec_reader_mmap_csvlite_get_header_single_seps(file_reader_mmap_
 	for ( ; p < phandle->eof && *p; ) {
 		if (*p == irs) {
 			*p = 0;
+
+			if (pstate->do_auto_line_term) {
+				if (p > phandle->sol && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			pstate->ilno++;
 			break;
@@ -352,6 +386,16 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps(file_reader_mmap_
 				return NULL;
 			}
 			*p = 0;
+
+			if (pstate->do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			pstate->ilno++;
 			saw_rs = TRUE;
@@ -538,13 +582,23 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps_implicit_header(f
 				return NULL;
 			}
 			*p = 0;
+
+			if (pstate->do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			pstate->ilno++;
 			saw_rs = TRUE;
 			break;
 		} else if (*p == ifs) {
 			*p = 0;
-			key = make_nidx_key(++idx, &free_flags);
+			key = low_int_to_string(++idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 			p++;
 			if (allow_repeat_ifs) {
@@ -562,7 +616,7 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_single_seps_implicit_header(f
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(++idx, &free_flags);
+	key = low_int_to_string(++idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
@@ -620,7 +674,7 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(fi
 			break;
 		} else if (streqn(p, ifs, ifslen)) {
 			*p = 0;
-			key = make_nidx_key(++idx, &free_flags);
+			key = low_int_to_string(++idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p += ifslen;
@@ -639,7 +693,7 @@ static lrec_t* lrec_reader_mmap_csvlite_get_record_multi_seps_implicit_header(fi
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(++idx, &free_flags);
+	key = low_int_to_string(++idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
