@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "lib/mlr_globals.h"
@@ -13,6 +22,7 @@ typedef struct _lrec_reader_mmap_dkvp_state_t {
 	int   ifslen;
 	int   ipslen;
 	int   allow_repeat_ifs;
+	int   do_auto_line_term;
 } lrec_reader_mmap_dkvp_state_t;
 
 static void    lrec_reader_mmap_dkvp_free(lrec_reader_t* preader);
@@ -34,11 +44,23 @@ lrec_reader_t* lrec_reader_mmap_dkvp_alloc(char* irs, char* ifs, char* ips, int
 	pstate->ifslen           = strlen(ifs);
 	pstate->ipslen           = strlen(ips);
 	pstate->allow_repeat_ifs = allow_repeat_ifs;
+	pstate->do_auto_line_term      = FALSE;
 
 	plrec_reader->pvstate       = (void*)pstate;
 	plrec_reader->popen_func    = file_reader_mmap_vopen;
 	plrec_reader->pclose_func   = file_reader_mmap_vclose;
-	if (pstate->irslen == 1) {
+	if (streq(irs, "auto")) {
+		// Auto means either lines end in "\n" or "\r\n" (LF or CRLF).  In
+		// either case the final character is "\n". Then for autodetect we
+		// simply check if there's a character in the line before the '\n', and
+		// if that is '\r'.
+		pstate->do_auto_line_term = TRUE;
+		pstate->irs = "\n";
+		pstate->irslen = 1;
+		plrec_reader->pprocess_func = (pstate->ifslen == 1 && pstate->ipslen == 1)
+			? lrec_reader_mmap_dkvp_process_single_irs_single_others
+			: lrec_reader_mmap_dkvp_process_single_irs_multi_others;
+	} else if (pstate->irslen == 1) {
 		plrec_reader->pprocess_func = (pstate->ifslen == 1 && pstate->ipslen == 1)
 			? lrec_reader_mmap_dkvp_process_single_irs_single_others
 			: lrec_reader_mmap_dkvp_process_single_irs_multi_others;
@@ -70,7 +92,7 @@ static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_single_others(void* pvst
 		return NULL;
 	else
 		return lrec_parse_mmap_dkvp_single_irs_single_others(phandle, pstate->irs[0], pstate->ifs[0], pstate->ips[0],
-			pstate->allow_repeat_ifs, pctx);
+			pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_multi_others(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -80,7 +102,7 @@ static lrec_t* lrec_reader_mmap_dkvp_process_single_irs_multi_others(void* pvsta
 		return NULL;
 	else
 		return lrec_parse_mmap_dkvp_single_irs_multi_others(phandle, pstate->irs[0], pstate->ifs, pstate->ips,
-			pstate->ifslen, pstate->ipslen, pstate->allow_repeat_ifs, pctx);
+			pstate->ifslen, pstate->ipslen, pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_single_others(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -105,7 +127,7 @@ static lrec_t* lrec_reader_mmap_dkvp_process_multi_irs_multi_others(void* pvstat
 
 // ----------------------------------------------------------------
 lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *phandle,
-	char irs, char ifs, char ips, int allow_repeat_ifs, context_t* pctx)
+	char irs, char ifs, char ips, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
 {
 	lrec_t* prec = lrec_unbacked_alloc();
 
@@ -126,6 +148,16 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 	for ( ; p < phandle->eof && *p; ) {
 		if (*p == irs) {
 			*p = 0;
+
+			if (do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			saw_rs = TRUE;
 			break;
@@ -139,7 +171,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 				// "a=".  Here we use the positional index as the key. This way
 				// DKVP is a generalization of NIDX.
 				char free_flags = NO_FREE;
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 			}
 			else {
 				lrec_put(prec, key, value, NO_FREE);
@@ -175,9 +207,9 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof)
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			else
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 		}
 		else {
 			if (value >= phandle->eof)
@@ -193,10 +225,10 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_single_others(file_reader_mmap_state_t *
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof) {
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			} else {
 				char* copy = mlr_alloc_string_from_char_range(value, phandle->eof - value);
-				lrec_put(prec, make_nidx_key(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
 			}
 		}
 		else {
@@ -247,7 +279,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 				// "a=".  Here we use the positional index as the key. This way
 				// DKVP is a generalization of NIDX.
 				char free_flags = NO_FREE;
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 			}
 			else {
 				lrec_put(prec, key, value, NO_FREE);
@@ -283,9 +315,9 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof)
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			else
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 		}
 		else {
 			if (value >= phandle->eof)
@@ -301,10 +333,10 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof) {
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			} else {
 				char* copy = mlr_alloc_string_from_char_range(value, phandle->eof - value);
-				lrec_put(prec, make_nidx_key(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
 			}
 		}
 		else {
@@ -321,7 +353,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_single_others(file_reader_mmap_state_t *p
 }
 
 lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *phandle, char irs, char* ifs, char* ips,
-	int ifslen, int ipslen, int allow_repeat_ifs, context_t* pctx)
+	int ifslen, int ipslen, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
 {
 	lrec_t* prec = lrec_unbacked_alloc();
 
@@ -342,6 +374,16 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 	for ( ; p < phandle->eof && *p; ) {
 		if (*p == irs) {
 			*p = 0;
+
+			if (do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			saw_rs = TRUE;
 			break;
@@ -355,7 +397,7 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 				// "a=".  Here we use the positional index as the key. This way
 				// DKVP is a generalization of NIDX.
 				char free_flags = NO_FREE;
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 			}
 			else {
 				lrec_put(prec, key, value, NO_FREE);
@@ -392,9 +434,9 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof)
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			else
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 		}
 		else {
 			if (value >= phandle->eof)
@@ -410,10 +452,10 @@ lrec_t* lrec_parse_mmap_dkvp_single_irs_multi_others(file_reader_mmap_state_t *p
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof) {
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			} else {
 				char* copy = mlr_alloc_string_from_char_range(value, phandle->eof - value);
-				lrec_put(prec, make_nidx_key(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
 			}
 		}
 		else {
@@ -464,7 +506,7 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 				// "a=".  Here we use the positional index as the key. This way
 				// DKVP is a generalization of NIDX.
 				char free_flags = NO_FREE;
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 			}
 			else {
 				lrec_put(prec, key, value, NO_FREE);
@@ -500,9 +542,9 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof)
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			else
-				lrec_put(prec, make_nidx_key(idx, &free_flags), value, free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), value, free_flags);
 		}
 		else {
 			if (value >= phandle->eof)
@@ -518,10 +560,10 @@ lrec_t* lrec_parse_mmap_dkvp_multi_irs_multi_others(file_reader_mmap_state_t *ph
 		if (*key == 0 || value <= key) {
 			char free_flags = NO_FREE;
 			if (value >= phandle->eof) {
-				lrec_put(prec, make_nidx_key(idx, &free_flags), "", free_flags);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), "", free_flags);
 			} else {
 				char* copy = mlr_alloc_string_from_char_range(value, phandle->eof - value);
-				lrec_put(prec, make_nidx_key(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
+				lrec_put(prec, low_int_to_string(idx, &free_flags), copy, free_flags | FREE_ENTRY_VALUE);
 			}
 		}
 		else {
