@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdlib.h>
 #include "lib/mlrutil.h"
 #include "input/file_reader_mmap.h"
@@ -9,6 +18,7 @@ typedef struct _lrec_reader_mmap_nidx_state_t {
 	int   irslen;
 	int   ifslen;
 	int   allow_repeat_ifs;
+	int   do_auto_line_term;
 } lrec_reader_mmap_nidx_state_t;
 
 static void    lrec_reader_mmap_nidx_free(lrec_reader_t* preader);
@@ -28,12 +38,24 @@ lrec_reader_t* lrec_reader_mmap_nidx_alloc(char* irs, char* ifs, int allow_repea
 	pstate->irslen                   = strlen(pstate->irs);
 	pstate->ifslen                   = strlen(pstate->ifs);
 	pstate->allow_repeat_ifs         = allow_repeat_ifs;
+	pstate->do_auto_line_term      = FALSE;
 
 	plrec_reader->pvstate       = (void*)pstate;
 	plrec_reader->popen_func    = file_reader_mmap_vopen;
 	plrec_reader->pclose_func   = file_reader_mmap_vclose;
 
-	if (pstate->irslen == 1) {
+	if (streq(irs, "auto")) {
+		// Auto means either lines end in "\n" or "\r\n" (LF or CRLF).  In
+		// either case the final character is "\n". Then for autodetect we
+		// simply check if there's a character in the line before the '\n', and
+		// if that is '\r'.
+		pstate->do_auto_line_term = TRUE;
+		pstate->irs = "\n";
+		pstate->irslen = 1;
+		plrec_reader->pprocess_func = (pstate->ifslen == 1)
+			? lrec_reader_mmap_nidx_process_single_irs_single_ifs
+			: lrec_reader_mmap_nidx_process_single_irs_multi_ifs;
+	} else if (pstate->irslen == 1) {
 		plrec_reader->pprocess_func = (pstate->ifslen == 1)
 			? lrec_reader_mmap_nidx_process_single_irs_single_ifs
 			: lrec_reader_mmap_nidx_process_single_irs_multi_ifs;
@@ -66,7 +88,7 @@ static lrec_t* lrec_reader_mmap_nidx_process_single_irs_single_ifs(void* pvstate
 		return NULL;
 	else
 		return lrec_parse_mmap_nidx_single_irs_single_ifs(phandle, pstate->irs[0], pstate->ifs[0],
-			pstate->allow_repeat_ifs);
+			pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_nidx_process_single_irs_multi_ifs(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -76,7 +98,7 @@ static lrec_t* lrec_reader_mmap_nidx_process_single_irs_multi_ifs(void* pvstate,
 		return NULL;
 	else
 		return lrec_parse_mmap_nidx_single_irs_multi_ifs(phandle, pstate->irs[0], pstate->ifs,
-			pstate->ifslen, pstate->allow_repeat_ifs);
+			pstate->ifslen, pstate->allow_repeat_ifs, pstate->do_auto_line_term, pctx);
 }
 
 static lrec_t* lrec_reader_mmap_nidx_process_multi_irs_single_ifs(void* pvstate, void* pvhandle, context_t* pctx) {
@@ -101,7 +123,7 @@ static lrec_t* lrec_reader_mmap_nidx_process_multi_irs_multi_ifs(void* pvstate,
 
 // ----------------------------------------------------------------
 lrec_t* lrec_parse_mmap_nidx_single_irs_single_ifs(file_reader_mmap_state_t *phandle,
-	char irs, char ifs, int allow_repeat_ifs)
+	char irs, char ifs, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
 {
 	lrec_t* prec = lrec_unbacked_alloc();
 
@@ -120,6 +142,16 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_single_ifs(file_reader_mmap_state_t *pha
 	for ( ; p < phandle->eof && *p; ) {
 		if (*p == irs) {
 			*p = 0;
+
+			if (do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			saw_rs = TRUE;
 			break;
@@ -127,7 +159,7 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_single_ifs(file_reader_mmap_state_t *pha
 			*p = 0;
 
 			idx++;
-			key = make_nidx_key(idx, &free_flags);
+			key = low_int_to_string(idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p++;
@@ -147,7 +179,7 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_single_ifs(file_reader_mmap_state_t *pha
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(idx, &free_flags);
+	key = low_int_to_string(idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
@@ -166,7 +198,7 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_single_ifs(file_reader_mmap_state_t *pha
 }
 
 lrec_t* lrec_parse_mmap_nidx_single_irs_multi_ifs(file_reader_mmap_state_t *phandle,
-	char irs, char* ifs, int ifslen, int allow_repeat_ifs)
+	char irs, char* ifs, int ifslen, int allow_repeat_ifs, int do_auto_line_term, context_t* pctx)
 {
 	lrec_t* prec = lrec_unbacked_alloc();
 
@@ -185,6 +217,16 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_multi_ifs(file_reader_mmap_state_t *phan
 	for ( ; p < phandle->eof && *p; ) {
 		if (*p == irs) {
 			*p = 0;
+
+			if (do_auto_line_term) {
+				if (p > line && p[-1] == '\r') {
+					p[-1] = 0;
+					context_set_autodetected_crlf(pctx);
+				} else {
+					context_set_autodetected_lf(pctx);
+				}
+			}
+
 			phandle->sol = p+1;
 			saw_rs = TRUE;
 			break;
@@ -192,7 +234,7 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_multi_ifs(file_reader_mmap_state_t *phan
 			*p = 0;
 
 			idx++;
-			key = make_nidx_key(idx, &free_flags);
+			key = low_int_to_string(idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p += ifslen;
@@ -212,7 +254,7 @@ lrec_t* lrec_parse_mmap_nidx_single_irs_multi_ifs(file_reader_mmap_state_t *phan
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(idx, &free_flags);
+	key = low_int_to_string(idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
@@ -257,7 +299,7 @@ lrec_t* lrec_parse_mmap_nidx_multi_irs_single_ifs(file_reader_mmap_state_t *phan
 			*p = 0;
 
 			idx++;
-			key = make_nidx_key(idx, &free_flags);
+			key = low_int_to_string(idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p++;
@@ -277,7 +319,7 @@ lrec_t* lrec_parse_mmap_nidx_multi_irs_single_ifs(file_reader_mmap_state_t *phan
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(idx, &free_flags);
+	key = low_int_to_string(idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
@@ -322,7 +364,7 @@ lrec_t* lrec_parse_mmap_nidx_multi_irs_multi_ifs(file_reader_mmap_state_t *phand
 			*p = 0;
 
 			idx++;
-			key = make_nidx_key(idx, &free_flags);
+			key = low_int_to_string(idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p += ifslen;
@@ -342,7 +384,7 @@ lrec_t* lrec_parse_mmap_nidx_multi_irs_multi_ifs(file_reader_mmap_state_t *phand
 	if (allow_repeat_ifs && *value == 0)
 		return prec;
 
-	key = make_nidx_key(idx, &free_flags);
+	key = low_int_to_string(idx, &free_flags);
 
 	if (saw_rs) {
 		// Easy and simple case: we read until end of line.  We zero-poked the irs to a null character to terminate the
