@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -32,9 +41,10 @@
 #define IFS_STRIDX           0x2003
 #define DQUOTE_STRIDX        0x2004
 #define DQUOTE_IRS_STRIDX    0x2005
-#define DQUOTE_IFS_STRIDX    0x2006
-#define DQUOTE_EOF_STRIDX    0x2007
-#define DQUOTE_DQUOTE_STRIDX 0x2008
+#define DQUOTE_IRS2_STRIDX   0x2006 // alternate line-ending for autodetect LF/CRLF
+#define DQUOTE_IFS_STRIDX    0x2007
+#define DQUOTE_EOF_STRIDX    0x2008
+#define DQUOTE_DQUOTE_STRIDX 0x2009
 
 // ----------------------------------------------------------------
 typedef struct _lrec_reader_stdio_csv_state_t {
@@ -46,12 +56,13 @@ typedef struct _lrec_reader_stdio_csv_state_t {
 	char* irs;
 	char* ifs_eof;
 	char* ifs;
+	int   do_auto_line_term;
 	char* dquote;
 	char* dquote_irs;
+	char* dquote_irs2;
 	char* dquote_ifs;
 	char* dquote_eof;
 	char* dquote_dquote;
-
 	int   dquotelen;
 
 	rslls_t*            pfields;
@@ -72,7 +83,8 @@ typedef struct _lrec_reader_stdio_csv_state_t {
 static void    lrec_reader_stdio_csv_free(lrec_reader_t* preader);
 static void    lrec_reader_stdio_csv_sof(void* pvstate, void* pvhandle);
 static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, context_t* pctx);
-static int     lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields);
+static int     lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields,
+	context_t* pctx);
 static lrec_t* paste_indices_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
 	context_t* pctx);
 static lrec_t* paste_header_and_data(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pdata_fields,
@@ -87,13 +99,18 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 	lrec_reader_stdio_csv_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_stdio_csv_state_t));
 	pstate->ilno          = 0LL;
 
+	pstate->do_auto_line_term = FALSE;
+	if (streq(irs, "auto")) {
+		irs = "\n";
+		pstate->do_auto_line_term = TRUE;
+	}
+
 	pstate->eof           = "\xff";
 	pstate->irs           = irs;
 	pstate->ifs           = ifs;
 	pstate->ifs_eof       = mlr_paste_2_strings(pstate->ifs, "\xff");
 	pstate->dquote        = "\"";
 
-	pstate->dquote_irs    = mlr_paste_2_strings("\"", pstate->irs);
 	pstate->dquote_ifs    = mlr_paste_2_strings("\"", pstate->ifs);
 	pstate->dquote_eof    = "\"\xff";
 	pstate->dquote_dquote = "\"\"";
@@ -108,6 +125,16 @@ lrec_reader_t* lrec_reader_stdio_csv_alloc(char* irs, char* ifs, int use_implici
 	parse_trie_add_string(pstate->pno_dquote_parse_trie, pstate->dquote,  DQUOTE_STRIDX);
 
 	pstate->pdquote_parse_trie = parse_trie_alloc();
+	if (pstate->do_auto_line_term) {
+		pstate->dquote_irs  = mlr_paste_2_strings("\"", "\n");
+		pstate->dquote_irs2 = mlr_paste_2_strings("\"", "\r\n");
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,  DQUOTE_IRS_STRIDX);
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs2, DQUOTE_IRS2_STRIDX);
+	} else {
+		pstate->dquote_irs  = mlr_paste_2_strings("\"", pstate->irs);
+		pstate->dquote_irs2 = NULL;
+		parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs, DQUOTE_IRS_STRIDX);
+	}
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->eof,           EOF_STRIDX);
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_irs,    DQUOTE_IRS_STRIDX);
 	parse_trie_add_string(pstate->pdquote_parse_trie, pstate->dquote_ifs,    DQUOTE_IFS_STRIDX);
@@ -151,6 +178,7 @@ static void lrec_reader_stdio_csv_free(lrec_reader_t* preader) {
 	sb_free(pstate->psb);
 	free(pstate->ifs_eof);
 	free(pstate->dquote_irs);
+	free(pstate->dquote_irs2);
 	free(pstate->dquote_ifs);
 	free(pstate);
 	free(preader);
@@ -168,7 +196,7 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 	lrec_reader_stdio_csv_state_t* pstate = pvstate;
 
 	if (pstate->expect_header_line_next) {
-		if (!lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields))
+		if (!lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx))
 			return NULL;
 		pstate->ilno++;
 
@@ -182,7 +210,14 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 			}
 			// Transfer pointer-free responsibility from the rslls to the
 			// header fields in the header keeper
-			slls_append(pheader_fields, pe->value, pe->free_flag);
+			if (string_starts_with(pe->value, "\xef\xbb\xbf")) {
+				// Strip UTF-8 BOM if any
+				slls_append(pheader_fields, mlr_strdup_or_die(&pe->value[3]), FREE_ENTRY_VALUE);
+				if (pe->free_flag & FREE_ENTRY_VALUE)
+					free(pe->value);
+			} else {
+				slls_append(pheader_fields, pe->value, pe->free_flag);
+			}
 			pe->free_flag = 0;
 		}
 		rslls_reset(pstate->pfields);
@@ -198,7 +233,7 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 
 		pstate->expect_header_line_next = FALSE;
 	}
-	int rc = lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields);
+	int rc = lrec_reader_stdio_csv_get_fields(pstate, pstate->pfields, pctx);
 	pstate->ilno++;
 	if (rc == FALSE) // EOF
 		return NULL;
@@ -211,10 +246,14 @@ static lrec_t* lrec_reader_stdio_csv_process(void* pvstate, void* pvhandle, cont
 	}
 }
 
-static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields) {
+static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstate, rslls_t* pfields,
+	context_t* pctx)
+{
 	int rc, stridx, matchlen, record_done, field_done;
 	peek_file_reader_t* pfr = pstate->pfr;
 	string_builder_t*   psb = pstate->psb;
+	char* field = NULL;
+	int field_length = 0;
 
 	if (pfr_peek_char(pfr) == (char)EOF) // char defaults to unsigned on some platforms
 		return FALSE;
@@ -256,7 +295,19 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 						field_done  = TRUE;
 						break;
 					case IRS_STRIDX: // end of record
-						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, 0);
+						field = sb_finish_with_length(psb, &field_length);
+
+						// The line-ending '\n' won't be included in the field buffer.
+						if (pstate->do_auto_line_term) {
+							if (field_length > 0 && field[field_length-1] == '\r') {
+								field[field_length-1] = 0;
+								context_set_autodetected_crlf(pctx);
+							} else {
+								context_set_autodetected_lf(pctx);
+							}
+						}
+
+						rslls_append(pfields, field, FREE_ENTRY_VALUE, 0);
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
@@ -288,6 +339,8 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 
 			// loop over characters in field
 			field_done = FALSE;
+			char* field = NULL;
+			int field_length = 0;
 			while (!field_done) {
 				pfr_buffer_by(pfr, pstate->pdquote_parse_trie->maxlen);
 
@@ -298,7 +351,7 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 				if (rc) {
 					switch(stridx) {
 					case EOF_STRIDX: // end of record
-						fprintf(stderr, "%s: imbalanced double-quote at line %lld.\n",
+						fprintf(stderr, "%s: unmatched double quote at line %lld.\n",
 							MLR_GLOBALS.bargv0, pstate->ilno);
 						exit(1);
 						break;
@@ -312,7 +365,21 @@ static int lrec_reader_stdio_csv_get_fields(lrec_reader_stdio_csv_state_t* pstat
 						field_done  = TRUE;
 						break;
 					case DQUOTE_IRS_STRIDX: // end of record
-						rslls_append(pfields, sb_finish(psb), FREE_ENTRY_VALUE, FIELD_QUOTED_ON_INPUT);
+					case DQUOTE_IRS2_STRIDX: // end of record
+
+						field = sb_finish_with_length(psb, &field_length);
+
+						// The line-ending '\n' won't be included in the field buffer.
+						if (pstate->do_auto_line_term) {
+							if (field_length > 0 && field[field_length-1] == '\r') {
+								field[field_length-1] = 0;
+								context_set_autodetected_crlf(pctx);
+							} else {
+								context_set_autodetected_lf(pctx);
+							}
+						}
+
+						rslls_append(pfields, field, FREE_ENTRY_VALUE, FIELD_QUOTED_ON_INPUT);
 						field_done  = TRUE;
 						record_done = TRUE;
 						break;
@@ -346,7 +413,7 @@ static lrec_t* paste_indices_and_data(lrec_reader_stdio_csv_state_t* pstate, rsl
 	for (rsllse_t* pd = pdata_fields->phead; pd != NULL; pd = pd->pnext) {
 		idx++;
 		char free_flags = pd->free_flag;
-		char* key = make_nidx_key(idx, &free_flags);
+		char* key = low_int_to_string(idx, &free_flags);
 		// Transfer pointer-free responsibility from the rslls to the lrec object
 		lrec_put_ext(prec, key, pd->value, free_flags, pd->quote_flag);
 		pd->free_flag = 0;
