@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "lib/mlr_globals.h"
@@ -39,12 +48,14 @@
 typedef struct _lrec_reader_stdio_csvlite_state_t {
 	long long  ifnr;
 	long long  ilno; // Line-level, not record-level as in context_t
-	char* irs;
-	char* ifs;
-	int   irslen;
-	int   ifslen;
-	int   allow_repeat_ifs;
-	int   use_implicit_header;
+	char*  irs;
+	char*  ifs;
+	int    irslen;
+	int    ifslen;
+	int    allow_repeat_ifs;
+	int    do_auto_line_term;
+	int    use_implicit_header;
+	size_t line_length;
 
 	int  expect_header_line_next;
 	header_keeper_t* pheader_keeper;
@@ -60,20 +71,33 @@ lrec_reader_t* lrec_reader_stdio_csvlite_alloc(char* irs, char* ifs, int allow_r
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_stdio_csvlite_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_stdio_csvlite_state_t));
-	pstate->ifnr                      = 0LL;
-	pstate->irs                       = irs;
-	pstate->ifs                       = ifs;
-	pstate->irslen                    = strlen(irs);
-	pstate->ifslen                    = strlen(ifs);
-	pstate->allow_repeat_ifs          = allow_repeat_ifs;
-	pstate->use_implicit_header       = use_implicit_header;
-	pstate->expect_header_line_next   = use_implicit_header ? FALSE : TRUE;
-	pstate->pheader_keeper            = NULL;
-	pstate->pheader_keepers           = lhmslv_alloc();
+	pstate->ifnr                    = 0LL;
+	pstate->irs                     = irs;
+	pstate->ifs                     = ifs;
+	pstate->irslen                  = strlen(irs);
+	pstate->ifslen                  = strlen(ifs);
+	pstate->allow_repeat_ifs        = allow_repeat_ifs;
+	pstate->do_auto_line_term       = FALSE;
+	pstate->use_implicit_header     = use_implicit_header;
+	// This is used to track nominal line length over the file read. Bootstrap with a default length.
+	pstate->line_length             = MLR_ALLOC_READ_LINE_INITIAL_SIZE;
+
+	pstate->expect_header_line_next = use_implicit_header  ? FALSE : TRUE;
+	pstate->pheader_keeper          = NULL;
+	pstate->pheader_keepers         = lhmslv_alloc();
 
 	plrec_reader->pvstate       = (void*)pstate;
 	plrec_reader->popen_func    = file_reader_stdio_vopen;
 	plrec_reader->pclose_func   = file_reader_stdio_vclose;
+	if (streq(irs, "auto")) {
+		// Auto means either lines end in "\n" or "\r\n" (LF or CRLF).  In
+		// either case the final character is "\n". Then for autodetect we
+		// simply check if there's a character in the line before the '\n', and
+		// if that is '\r'.
+		pstate->irs = "\n";
+		pstate->irslen = 1;
+		pstate->do_auto_line_term = TRUE;
+	}
 	plrec_reader->pprocess_func = lrec_reader_stdio_csvlite_process;
 	plrec_reader->psof_func     = lrec_reader_stdio_sof;
 	plrec_reader->pfree_func    = lrec_reader_stdio_csvlite_free;
@@ -110,8 +134,11 @@ static lrec_t* lrec_reader_stdio_csvlite_process(void* pvstate, void* pvhandle,
 		if (pstate->expect_header_line_next) {
 			while (TRUE) {
 				char* hline = (pstate->irslen == 1)
-					? mlr_get_cline(input_stream, pstate->irs[0])
-					: mlr_get_sline(input_stream, pstate->irs, pstate->irslen);
+					? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+						&pstate->line_length, pstate->do_auto_line_term, pctx)
+					: mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
+						&pstate->line_length);
+
 				if (hline == NULL) // EOF
 					return NULL;
 				pstate->ilno++;
@@ -150,8 +177,11 @@ static lrec_t* lrec_reader_stdio_csvlite_process(void* pvstate, void* pvhandle,
 		}
 
 		char* line = (pstate->irslen == 1)
-			? mlr_get_cline(input_stream, pstate->irs[0])
-			: mlr_get_sline(input_stream, pstate->irs, pstate->irslen);
+			? mlr_alloc_read_line_single_delimiter(input_stream, pstate->irs[0],
+				&pstate->line_length, pstate->do_auto_line_term, pctx)
+			: mlr_alloc_read_line_multiple_delimiter(input_stream, pstate->irs, pstate->irslen,
+				&pstate->line_length);
+
 		if (line == NULL) // EOF
 			return NULL;
 		pstate->ilno++;
@@ -170,15 +200,15 @@ static lrec_t* lrec_reader_stdio_csvlite_process(void* pvstate, void* pvhandle,
 					? lrec_parse_stdio_csvlite_data_line_single_ifs_implicit_header(
 						pstate->pheader_keeper, pctx->filename, pstate->ilno, line,
 						pstate->ifs[0], pstate->allow_repeat_ifs)
-					:  lrec_parse_stdio_csvlite_data_line_single_ifs(pstate->pheader_keeper, pctx->filename, pstate->ilno, line,
-						pstate->ifs[0], pstate->allow_repeat_ifs);
+					: lrec_parse_stdio_csvlite_data_line_single_ifs(pstate->pheader_keeper, pctx->filename,
+						pstate->ilno, line, pstate->ifs[0], pstate->allow_repeat_ifs);
 			} else {
 				return pstate->use_implicit_header
 					? lrec_parse_stdio_csvlite_data_line_multi_ifs_implicit_header(
 						pstate->pheader_keeper, pctx->filename, pstate->ilno, line,
 						pstate->ifs, pstate->ifslen, pstate->allow_repeat_ifs)
-					: lrec_parse_stdio_csvlite_data_line_multi_ifs(pstate->pheader_keeper, pctx->filename, pstate->ilno, line,
-						pstate->ifs, pstate->ifslen, pstate->allow_repeat_ifs);
+					: lrec_parse_stdio_csvlite_data_line_multi_ifs(pstate->pheader_keeper, pctx->filename,
+						pstate->ilno, line, pstate->ifs, pstate->ifslen, pstate->allow_repeat_ifs);
 			}
 		}
 	}
@@ -380,7 +410,7 @@ lrec_t* lrec_parse_stdio_csvlite_data_line_single_ifs_implicit_header(header_kee
 		if (*p == ifs) {
 			*p = 0;
 
-			key = make_nidx_key(++idx, &free_flags);
+			key = low_int_to_string(++idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p++;
@@ -396,7 +426,7 @@ lrec_t* lrec_parse_stdio_csvlite_data_line_single_ifs_implicit_header(header_kee
 	if (allow_repeat_ifs && *value == 0) {
 		; // OK
 	} else {
-		key = make_nidx_key(++idx, &free_flags);
+		key = low_int_to_string(++idx, &free_flags);
 		lrec_put(prec, key, value, NO_FREE);
 		lrec_put(prec, key, value, free_flags);
 	}
@@ -422,7 +452,7 @@ lrec_t* lrec_parse_stdio_csvlite_data_line_multi_ifs_implicit_header(header_keep
 	for ( ; *p; ) {
 		if (streqn(p, ifs, ifslen)) {
 			*p = 0;
-			key = make_nidx_key(++idx, &free_flags);
+			key = low_int_to_string(++idx, &free_flags);
 			lrec_put(prec, key, value, free_flags);
 
 			p += ifslen;
@@ -438,7 +468,7 @@ lrec_t* lrec_parse_stdio_csvlite_data_line_multi_ifs_implicit_header(header_keep
 	if (allow_repeat_ifs && *value == 0) {
 		; // OK
 	} else {
-		key = make_nidx_key(++idx, &free_flags);
+		key = low_int_to_string(++idx, &free_flags);
 		lrec_put(prec, key, value, free_flags);
 	}
 
