@@ -1,3 +1,12 @@
+// ================================================================
+// Note: there are multiple process methods with a lot of code duplication.
+// This is intentional. Much of Miller's measured processing time is in the
+// lrec-reader process methods. This is code which needs to execute on every
+// byte of input and even moving a single runtime if-statement into a
+// function-pointer assignment at alloc time can have noticeable effects on
+// performance (5-10% in some cases).
+// ================================================================
+
 // ================================================================
 // This has at present a lot of code duplication with lrec_reader_mmap_json.
 // This is because we read the entire input file into memory and get a pointer
@@ -26,6 +35,9 @@ typedef struct _lrec_reader_stdio_json_state_t {
 	sllv_t* ptop_level_json_objects;
 	sllv_t* precords;
 	char* input_json_flatten_separator;
+	json_array_ingest_t json_array_ingest;
+	int do_auto_line_term;
+	char* detected_line_term;
 } lrec_reader_stdio_json_state_t;
 
 static void    lrec_reader_stdio_json_free(lrec_reader_t* preader);
@@ -33,13 +45,22 @@ static void    lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle);
 static lrec_t* lrec_reader_stdio_json_process(void* pvstate, void* pvhandle, context_t* pctx);
 
 // ----------------------------------------------------------------
-lrec_reader_t* lrec_reader_stdio_json_alloc(char* input_json_flatten_separator) {
+lrec_reader_t* lrec_reader_stdio_json_alloc(char* input_json_flatten_separator, json_array_ingest_t json_array_ingest,
+	char* line_term)
+{
 	lrec_reader_t* plrec_reader = mlr_malloc_or_die(sizeof(lrec_reader_t));
 
 	lrec_reader_stdio_json_state_t* pstate = mlr_malloc_or_die(sizeof(lrec_reader_stdio_json_state_t));
 	pstate->ptop_level_json_objects      = sllv_alloc();
 	pstate->precords                     = sllv_alloc();
 	pstate->input_json_flatten_separator = input_json_flatten_separator;
+	pstate->json_array_ingest            = json_array_ingest;
+	pstate->do_auto_line_term            = FALSE;
+	pstate->detected_line_term           = "\n"; // xxx adapt to MLR_GLOBALS/ctx-const for Windows port
+
+	if (streq(line_term, "auto")) {
+		pstate->do_auto_line_term = TRUE;
+	}
 
 	plrec_reader->pvstate       = (void*)pstate;
 	plrec_reader->popen_func    = file_ingestor_stdio_vopen;
@@ -106,8 +127,24 @@ static void lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle) {
 
 	json_char* item_start = json_input;
 	int length = phandle->eof - phandle->sof;
+	char* detected_line_term = NULL;
+
+	if (pstate->do_auto_line_term) {
+		// Find the first line-ending sequence (if any): LF or CRLF.
+		for (char* p = phandle->sof; p < phandle->eof; p++) {
+			if (p[0] == '\n') {
+				if (p > phandle->sof && p[-1] == '\r') {
+					detected_line_term = "\r\n";
+				} else {
+					detected_line_term = "\n";
+				}
+				break;
+			}
+		}
+	}
 
 	while (TRUE) {
+
 		parsed_top_level_json = json_parse(item_start, length, error_buf, &item_start);
 
 		if (parsed_top_level_json == NULL) {
@@ -117,7 +154,12 @@ static void lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle) {
 
 		// The lrecs have their string pointers pointing into the parsed-JSON objects (for
 		// efficiency) so it's important we not free the latter until our free method.
-		reference_json_objects_as_lrecs(pstate->precords, parsed_top_level_json, pstate->input_json_flatten_separator);
+		if (!reference_json_objects_as_lrecs(pstate->precords, parsed_top_level_json,
+			pstate->input_json_flatten_separator, pstate->json_array_ingest))
+		{
+			fprintf(stderr, "%s: Unable to parse JSON data.\n", MLR_GLOBALS.bargv0);
+			exit(1);
+		}
 
 		if (item_start == NULL)
 			break;
@@ -125,11 +167,25 @@ static void lrec_reader_stdio_json_sof(void* pvstate, void* pvhandle) {
 			break;
 		length -= (item_start - json_input);
 		json_input = item_start;
+		// json_parse goes up to the '\r' or '\n' (whichever is found first) on the first
+		// parse, then keeps going from there on the next. E.g. in the CRLF case it
+		// consumes the CR at the end of the first read and consumes the LF at the start
+		// of the second, and so on. After the very last parse, we need to here consume
+		// the final '\n' which is (by itself) a parse error.
+		if (length == 1 && *(char*)json_input == '\n') {
+			break;
+		}
+	}
+	if (detected_line_term != NULL) {
+		pstate->detected_line_term = detected_line_term;
 	}
 }
 
 // ----------------------------------------------------------------
 static lrec_t* lrec_reader_stdio_json_process(void* pvstate, void* pvhandle, context_t* pctx) {
 	lrec_reader_stdio_json_state_t* pstate = pvstate;
+	if (pstate->do_auto_line_term) {
+		context_set_autodetected_line_term(pctx, pstate->detected_line_term);
+	}
 	return sllv_pop(pstate->precords);
 }
