@@ -2,10 +2,46 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <sys/time.h>
 #include <sys/stat.h>
 #include "lib/mlrutil.h"
 #include "lib/mlr_globals.h"
+#include "lib/free_flags.h"
+
+// ----------------------------------------------------------------
+void mlr_internal_coding_error(char* file, int line) {
+	fprintf(stderr, "%s: internal coding error detected in file %s at line %d.\n",
+		MLR_GLOBALS.bargv0, file, line);
+	exit(1);
+}
+
+void mlr_internal_coding_error_if(int v, char* file, int line) {
+	if (v) {
+		mlr_internal_coding_error(file, line);
+	}
+}
+
+void mlr_internal_coding_error_unless(int v, char* file, int line) {
+	if (!v) {
+		mlr_internal_coding_error(file, line);
+	}
+}
+
+// ----------------------------------------------------------------
+char* mlr_strmsep(char **pstring, const char *sep, int seplen) {
+	char* string = *pstring;
+	if (string == NULL) {
+		return NULL;
+	}
+	char* pnext = strstr(string, sep);
+	if (pnext == NULL) {
+		*pstring = NULL;
+		return string;
+	} else {
+		*pnext = 0;
+		*pstring = pnext + seplen;
+		return string;
+	}
+}
 
 // ----------------------------------------------------------------
 int mlr_bsearch_double_for_insert(double* array, int size, double value) {
@@ -48,23 +84,15 @@ int mlr_bsearch_double_for_insert(double* array, int size, double value) {
 	return lo;
 }
 
-// ----------------------------------------------------------------
-// seconds since the epoch
-double get_systime() {
-	struct timeval tv = { .tv_sec = 0, .tv_usec = 0 };
-	(void)gettimeofday(&tv, NULL);
-	return (double)tv.tv_sec + (double)tv.tv_usec * 1e-6;
-}
-
 // ----------------------------------------------------------------
 void* mlr_malloc_or_die(size_t size) {
 	void* p = malloc(size);
 	if (p == NULL) {
-		fprintf(stderr, "malloc(%lu) failed.\n", (unsigned long)size);
+		fprintf(stderr, "malloc(%llu) failed.\n", (unsigned long long)size);
 		exit(1);
 	}
 #ifdef MLR_MALLOC_TRACE
-	fprintf(stderr, "MALLOC size=%d,p=%p\n", (int)size, p);
+	fprintf(stderr, "MALLOC size=%llu,p=%p\n", (unsigned long long)size, p);
 #endif
 	return p;
 }
@@ -73,15 +101,26 @@ void* mlr_malloc_or_die(size_t size) {
 void* mlr_realloc_or_die(void *optr, size_t size) {
 	void* nptr = realloc(optr, size);
 	if (nptr == NULL) {
-		fprintf(stderr, "realloc(%lu) failed.\n", (unsigned long)size);
+		fprintf(stderr, "realloc(%llu) failed.\n", (unsigned long long)size);
 		exit(1);
 	}
 #ifdef MLR_MALLOC_TRACE
-	fprintf(stderr, "REALLOC size=%d,p=%p\n", (int)size, nptr);
+	fprintf(stderr, "REALLOC size=%llu,p=%p\n", (unsigned long long)size, nptr);
 #endif
 	return nptr;
 }
 
+// ----------------------------------------------------------------
+char * mlr_strdup_quoted_or_die(const char *s1) {
+	int len = strlen(s1);
+	char* s2 = mlr_malloc_or_die(len+3);
+	s2[0] = '"';
+	strcpy(&s2[1], s1);
+	s2[len+1] = '"';
+	s2[len+2] = 0;
+	return s2;
+}
+
 // ----------------------------------------------------------------
 // The caller should free the return value from each of these.
 
@@ -182,6 +221,32 @@ int mlr_try_int_from_string(char* string, long long* pval) {
 	return 1;
 }
 
+// ----------------------------------------------------------------
+static char* low_int_to_string_data[] = {
+	"0",   "1",  "2",  "3",  "4",  "5",  "6",  "7",  "8",  "9",
+	"10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
+	"20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
+	"30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
+	"40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
+	"50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
+	"60", "61", "62", "63", "64", "65", "66", "67", "68", "69",
+	"70", "71", "72", "73", "74", "75", "76", "77", "78", "79",
+	"80", "81", "82", "83", "84", "85", "86", "87", "88", "89",
+	"90", "91", "92", "93", "94", "95", "96", "97", "98", "99", "100"
+};
+
+char* low_int_to_string(int idx, char* pfree_flags) {
+	if ((0 <= idx) && (idx <= 100)) {
+		*pfree_flags = 0;
+		return low_int_to_string_data[idx];
+	} else {
+		char buf[32];
+		sprintf(buf, "%d", idx);
+		*pfree_flags = FREE_ENTRY_KEY;
+		return mlr_strdup_or_die(buf);
+	}
+}
+
 // ----------------------------------------------------------------
 char* mlr_paste_2_strings(char* s1, char* s2) {
 	int n1 = strlen(s1);
@@ -255,25 +320,6 @@ int mlr_string_pair_hash_func(char* str1, char* str2) {
 	return (int)hash;
 }
 
-// ----------------------------------------------------------------
-// See the GNU timegm manpage -- this is what it does.
-time_t mlr_timegm(struct tm* ptm) {
-	time_t ret;
-	char* tz;
-
-	tz = getenv("TZ");
-	setenv("TZ", "GMT0", 1);
-	tzset();
-	ret = mktime(ptm);
-	if (tz) {
-		setenv("TZ", tz, 1);
-	} else {
-		unsetenv("TZ");
-	}
-	tzset();
-	return ret;
-}
-
 // ----------------------------------------------------------------
 // 0x00-0x7f (MSB is 0) are ASCII and printable.
 // 0x80-0xbf (MSBs are 10) are continuation characters and don't add to printable length.
@@ -317,12 +363,13 @@ int mlr_imax2(int a, int b) {
 }
 
 // ----------------------------------------------------------------
-// This is inefficient. It's quite fine for call-once, small-n use.
-
-int power_of_two_ceil(int n) {
-	while (n&(n-1))
-		n++;
-	return n;
+int power_of_two_above(int n) {
+	n |= (n >> 1);
+	n |= (n >> 2);
+	n |= (n >> 4);
+	n |= (n >> 8);
+	n |= (n >> 16);
+	return(n+1);
 }
 
 // ----------------------------------------------------------------
@@ -517,28 +564,21 @@ char* read_fp_into_memory(FILE* fp, size_t* psize) {
 }
 
 // ----------------------------------------------------------------
-char* alloc_comment_strip(char* input) {
-	char* output = mlr_strdup_or_die(input);
-	char* q = output;
-	int copying = TRUE;
+char* alloc_suffixed_temp_file_name(char* filename) {
+	const int suffix_length = 6;
+	static char bag[] = "abcdefghijklmnopqrstuvwxyz" "ABCDEFGHIJKLMNOPQRSTUVWXYZ" "0123456789";
+	const static int bag_length = sizeof(bag) - 1;
 
-	for (char* p = input; *p; p++) {
-		if (copying) {
-			if (*p == '#') {
-				copying = FALSE;
-			} else {
-				*q = *p;
-				q++;
-			}
-		} else {
-			if (*p == '\n') {
-				copying = TRUE;
-				*q = *p;
-				q++;
-			}
-		}
+	char* output = mlr_malloc_or_die(strlen(filename) + 2 + suffix_length);
+
+	int rand_start_index = sprintf(output, "%s.", filename);
+	char* rand_start_ptr = &output[rand_start_index];
+
+	int i = 0;
+	for ( ; i < suffix_length; i++) {
+		rand_start_ptr[i] = bag[get_mtrand_int32() % bag_length];
 	}
+	rand_start_ptr[i] = 0;
 
-	*q = 0;
 	return output;
 }
