@@ -5,6 +5,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include "mtrand.h"
 
 #define TRUE  1
 #define FALSE 0
@@ -12,6 +13,14 @@
 
 //#define MLR_MALLOC_TRACE
 
+// ----------------------------------------------------------------
+#define MLR_INTERNAL_CODING_ERROR() mlr_internal_coding_error(__FILE__, __LINE__)
+#define MLR_INTERNAL_CODING_ERROR_IF(v) mlr_internal_coding_error_if(v, __FILE__, __LINE__)
+#define MLR_INTERNAL_CODING_ERROR_UNLESS(v) mlr_internal_coding_error_unless(v, __FILE__, __LINE__)
+void mlr_internal_coding_error(char* file, int line);
+void mlr_internal_coding_error_if(int v, char* file, int line);
+void mlr_internal_coding_error_unless(int v, char* file, int line);
+
 // ----------------------------------------------------------------
 //int mlr_canonical_mod(int a, int n);
 static inline int mlr_canonical_mod(int a, int n) {
@@ -65,10 +74,12 @@ static inline int streqn(char* a, char* b, int n) {
 }
 
 // ----------------------------------------------------------------
-int mlr_bsearch_double_for_insert(double* array, int size, double value);
+// Like strsep but the sep argument is a multi-character delimiter,
+// not a set of single-character delimiters.
+char* mlr_strmsep(char **pstring, const char *sep, int seplen);
 
-// seconds since the epoch
-double get_systime();
+// ----------------------------------------------------------------
+int mlr_bsearch_double_for_insert(double* array, int size, double value);
 
 void*  mlr_malloc_or_die(size_t size);
 void*  mlr_realloc_or_die(void *ptr, size_t size);
@@ -83,6 +94,7 @@ static inline char * mlr_strdup_or_die(const char *s1) {
 #endif
 	return s2;
 }
+char * mlr_strdup_quoted_or_die(const char *s1);
 
 // The caller should free the return values from each of these.
 char* mlr_alloc_string_from_double(double value, char* fmt);
@@ -100,6 +112,10 @@ long long mlr_int_from_string_or_die(char* string);
 int    mlr_try_float_from_string(char* string, double* pval);
 int    mlr_try_int_from_string(char* string, long long* pval);
 
+// For small integers (as of this writing, 0 .. 100) returns a static string representation.
+// For other values, returns a dynamically allocated string representation.
+char* low_int_to_string(int idx, char* pfree_flags);
+
 // Inefficient and intended for call-rarely use. The caller should free the return values.
 char* mlr_paste_2_strings(char* s1, char* s2);
 char* mlr_paste_3_strings(char* s1, char* s2, char* s3);
@@ -109,9 +125,6 @@ char* mlr_paste_5_strings(char* s1, char* s2, char* s3, char* s4, char* s5);
 int mlr_string_hash_func(char *str);
 int mlr_string_pair_hash_func(char* str1, char* str2);
 
-// portable timegm replacement
-time_t mlr_timegm (struct tm *ptm);
-
 int strlen_for_utf8_display(char* str);
 int string_starts_with(char* string, char* prefix);
 // If pstrlen is non-null, after return it will contain strlen(string) for
@@ -119,7 +132,7 @@ int string_starts_with(char* string, char* prefix);
 int string_ends_with(char* string, char* suffix, int* pstringlen);
 
 int mlr_imax2(int a, int b);
-int power_of_two_ceil(int n);
+int power_of_two_above(int n);
 
 // The caller should free the return value. Maps two-character sequences such as
 // "\t", "\n", "\\" to single characters such as tab, newline, backslash, etc.
@@ -159,8 +172,7 @@ char* read_file_into_memory(char* filename, size_t* psize);
 // The caller should free the return value.
 char* read_fp_into_memory(FILE* fp, size_t* psize);
 
-// Returns a copy of the input string with pound-sign to newline elided.
-// Does not modify the input. The caller should free the output.
-char* alloc_comment_strip(char* input);
+// Returns a copy of the filename with random characters attached to the end.
+char* alloc_suffixed_temp_file_name(char* filename);
 
 #endif // MLRUTIL_H
