@@ -11,7 +11,6 @@
 #include "containers/slls.h"
 #include "containers/lhmslv.h"
 #include "containers/lhmsv.h"
-#include "containers/lhmsi.h"
 #include "containers/mixutil.h"
 #include "containers/mlrval.h"
 #include "mapping/mappers.h"
@@ -61,7 +60,7 @@ static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv
 static mapper_t* mapper_merge_fields_alloc(slls_t* paccumulator_names, merge_by_t do_which,
 	slls_t* pvalue_field_names, char* output_field_basename, int allow_int_float, int do_interpolated_percentiles,
 	int keep_input_fields);
-static void      mapper_merge_fields_free(mapper_t* pmapper);
+static void      mapper_merge_fields_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_merge_fields_process_by_name_list(lrec_t* pinrec, context_t* pctx, void* pvstate);
 static sllv_t*   mapper_merge_fields_process_by_name_regex(lrec_t* pinrec, context_t* pctx, void* pvstate);
 static sllv_t*   mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, context_t* pctx, void* pvstate);
@@ -70,7 +69,8 @@ static sllv_t*   mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, conte
 mapper_setup_t mapper_merge_fields_setup = {
 	.verb        = "merge-fields",
 	.pusage_func = mapper_merge_fields_usage,
-	.pparse_func = mapper_merge_fields_parse_cli
+	.pparse_func = mapper_merge_fields_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -90,10 +90,16 @@ static void mapper_merge_fields_usage(FILE* o, char* argv0, char* verb) {
 	fprintf(o, "            after removing substrings will be accumulated together. Please see\n");
 	fprintf(o, "            examples below.\n");
 	fprintf(o, "-i          Use interpolated percentiles, like R's type=7; default like type=1.\n");
+	fprintf(o, "            Not sensical for string-valued fields.\n");
 	fprintf(o, "-o {name}   Output field basename for -f/-r.\n");
 	fprintf(o, "-k          Keep the input fields which contributed to the output statistics;\n");
 	fprintf(o, "            the default is to omit them.\n");
 	fprintf(o, "-F          Computes integerable things (e.g. count) in floating point.\n");
+	fprintf(o, "\n");
+	fprintf(o, "String-valued data make sense unless arithmetic on them is required,\n");
+	fprintf(o, "e.g. for sum, mean, interpolated percentiles, etc. In case of mixed data,\n");
+	fprintf(o, "numbers are less than strings.\n");
+	fprintf(o, "\n");
 	fprintf(o, "Example input data: \"a_in_x=1,a_out_x=2,b_in_y=4,b_out_x=8\".\n");
 	fprintf(o, "Example: %s %s -a sum,count -f a_in_x,a_out_x -o foo\n", argv0, verb);
 	fprintf(o, "  produces \"b_in_y=4,b_out_x=8,foo_sum=3,foo_count=2\" since \"a_in_x,a_out_x\" are\n");
@@ -128,8 +134,8 @@ static mapper_t* mapper_merge_fields_parse_cli(int* pargi, int argc, char** argv
 				mapper_merge_fields_usage(stderr, argv[0], verb);
 				return NULL;
 			}
-			if (pvalue_field_names != NULL)
-				slls_free(pvalue_field_names);
+			if (paccumulator_names != NULL)
+				slls_free(paccumulator_names);
 			paccumulator_names = slls_from_line(argv[argi+1], ',', FALSE);
 			argi += 2;
 
@@ -244,7 +250,7 @@ static mapper_t* mapper_merge_fields_alloc(slls_t* paccumulator_names, merge_by_
 	return pmapper;
 }
 
-static void mapper_merge_fields_free(mapper_t* pmapper) {
+static void mapper_merge_fields_free(mapper_t* pmapper, context_t* _) {
 	mapper_merge_fields_state_t* pstate = pmapper->pvstate;
 	slls_free(pstate->paccumulator_names);
 	slls_free(pstate->pvalue_field_names);
@@ -511,6 +517,8 @@ static sllv_t* mapper_merge_fields_process_by_collapsing(lrec_t* pinrec, context
 					pa = pa->pnext;
 				}
 				free(short_name);
+			} else {
+				free(short_name);
 			}
 		}
 		if (!matched)
