@@ -1,6 +1,6 @@
 #include "lib/mlrutil.h"
 #include "containers/lrec.h"
-#include "containers/mlrval.h"
+#include "containers/mvfuncs.h"
 #include "containers/slls.h"
 #include "containers/mixutil.h"
 #include "mapping/mappers.h"
@@ -13,7 +13,7 @@ static void      mapper_sec2gmtdate_usage(FILE* o, char* argv0, char* verb);
 static mapper_t* mapper_sec2gmtdate_parse_cli(int* pargi, int argc, char** argv,
 	cli_reader_opts_t* _, cli_writer_opts_t* __);
 static mapper_t* mapper_sec2gmtdate_alloc(slls_t* pfield_names);
-static void      mapper_sec2gmtdate_free(mapper_t* pmapper);
+static void      mapper_sec2gmtdate_free(mapper_t* pmapper, context_t* _);
 static sllv_t*   mapper_sec2gmtdate_process(lrec_t* pinrec, context_t* pctx, void* pvstate);
 
 // ----------------------------------------------------------------
@@ -21,6 +21,7 @@ mapper_setup_t mapper_sec2gmtdate_setup = {
 	.verb = "sec2gmtdate",
 	.pusage_func = mapper_sec2gmtdate_usage,
 	.pparse_func = mapper_sec2gmtdate_parse_cli,
+	.ignores_input = FALSE,
 };
 
 // ----------------------------------------------------------------
@@ -65,7 +66,7 @@ static mapper_t* mapper_sec2gmtdate_alloc(slls_t* pfield_names)
 	return pmapper;
 }
 
-static void mapper_sec2gmtdate_free(mapper_t* pmapper) {
+static void mapper_sec2gmtdate_free(mapper_t* pmapper, context_t* _) {
 	mapper_sec2gmtdate_state_t* pstate = pmapper->pvstate;
 	slls_free(pstate->pfield_names);
 	free(pstate);
