@@ -12,6 +12,7 @@ extern mapper_setup_t mapper_count_distinct_setup;
 extern mapper_setup_t mapper_cut_setup;
 extern mapper_setup_t mapper_decimate_setup;
 extern mapper_setup_t mapper_filter_setup;
+extern mapper_setup_t mapper_fraction_setup;
 extern mapper_setup_t mapper_grep_setup;
 extern mapper_setup_t mapper_group_by_setup;
 extern mapper_setup_t mapper_group_like_setup;
@@ -20,7 +21,9 @@ extern mapper_setup_t mapper_head_setup;
 extern mapper_setup_t mapper_histogram_setup;
 extern mapper_setup_t mapper_join_setup;
 extern mapper_setup_t mapper_label_setup;
+extern mapper_setup_t mapper_least_frequent_setup;
 extern mapper_setup_t mapper_merge_fields_setup;
+extern mapper_setup_t mapper_most_frequent_setup;
 extern mapper_setup_t mapper_nest_setup;
 extern mapper_setup_t mapper_nothing_setup;
 extern mapper_setup_t mapper_put_setup;
@@ -32,6 +35,7 @@ extern mapper_setup_t mapper_reshape_setup;
 extern mapper_setup_t mapper_sample_setup;
 extern mapper_setup_t mapper_sec2gmt_setup;
 extern mapper_setup_t mapper_sec2gmtdate_setup;
+extern mapper_setup_t mapper_seqgen_setup;
 extern mapper_setup_t mapper_shuffle_setup;
 extern mapper_setup_t mapper_sort_setup;
 extern mapper_setup_t mapper_stats1_setup;
@@ -42,5 +46,9 @@ extern mapper_setup_t mapper_tail_setup;
 extern mapper_setup_t mapper_tee_setup;
 extern mapper_setup_t mapper_top_setup;
 extern mapper_setup_t mapper_uniq_setup;
+extern mapper_setup_t mapper_unsparsify_setup;
+
+// Construction is in mlrcli.c.
+void mapper_chain_free(sllv_t* pmapper_chain, context_t* pctx);
 
 #endif // MAPPERS_H
