@@ -8,9 +8,9 @@
 #include "containers/lhmslv.h"
 #include "containers/lhmsv.h"
 #include "containers/lhmss.h"
-#include "containers/lhmsi.h"
+#include "containers/lhmsll.h"
 #include "containers/percentile_keeper.h"
-#include "containers/mlrval.h"
+#include "containers/mvfuncs.h"
 #include "mapping/stats1_accumulators.h"
 
 // ----------------------------------------------------------------
@@ -70,6 +70,8 @@ stats1_acc_t* make_stats1_acc(char* value_field_name, char* stats1_acc_name, int
 }
 
 int is_percentile_acc_name(char* stats1_acc_name) {
+	if (streq(stats1_acc_name, "median"))
+		return TRUE;
 	double percentile;
 	// sscanf(stats1_acc_name, "p%lf", &percentile) allows "p74x" et al. which isn't ok.
 	if (stats1_acc_name[0] != 'p')
@@ -131,16 +133,16 @@ stats1_acc_t* stats1_count_alloc(char* value_field_name, char* stats1_acc_name,
 
 // ----------------------------------------------------------------
 typedef struct _stats1_mode_state_t {
-	lhmsi_t* pcounts_for_value;
+	lhmsll_t* pcounts_for_value;
 	char* output_field_name;
 } stats1_mode_state_t;
 // mode on strings: "1" and "1.0" and "1.0000" are distinct text.
 static void stats1_mode_singest(void* pvstate, char* val) {
 	stats1_mode_state_t* pstate = pvstate;
-	lhmsie_t* pe = lhmsi_get_entry(pstate->pcounts_for_value, val);
+	lhmslle_t* pe = lhmsll_get_entry(pstate->pcounts_for_value, val);
 	if (pe == NULL) {
-		// lhmsi does a strdup so we needn't.
-		lhmsi_put(pstate->pcounts_for_value, mlr_strdup_or_die(val), 1, FREE_ENTRY_KEY);
+		// lhmsll does a strdup so we needn't.
+		lhmsll_put(pstate->pcounts_for_value, mlr_strdup_or_die(val), 1, FREE_ENTRY_KEY);
 	} else {
 		pe->value++;
 	}
@@ -149,7 +151,7 @@ static void stats1_mode_emit(void* pvstate, char* value_field_name, char* stats1
 	stats1_mode_state_t* pstate = pvstate;
 	int max_count = 0;
 	char* max_key = "";
-	for (lhmsie_t* pe = pstate->pcounts_for_value->phead; pe != NULL; pe = pe->pnext) {
+	for (lhmslle_t* pe = pstate->pcounts_for_value->phead; pe != NULL; pe = pe->pnext) {
 		int count = pe->value;
 		if (count > max_count) {
 			max_key = pe->key;
@@ -157,13 +159,14 @@ static void stats1_mode_emit(void* pvstate, char* value_field_name, char* stats1
 		}
 	}
 	if (copy_data)
-		lrec_put(poutrec, mlr_strdup_or_die(pstate->output_field_name), mlr_strdup_or_die(max_key), FREE_ENTRY_KEY|FREE_ENTRY_VALUE);
+		lrec_put(poutrec, mlr_strdup_or_die(pstate->output_field_name), mlr_strdup_or_die(max_key),
+			FREE_ENTRY_KEY|FREE_ENTRY_VALUE);
 	else
 		lrec_put(poutrec, pstate->output_field_name, max_key, NO_FREE);
 }
 static void stats1_mode_free(stats1_acc_t* pstats1_acc) {
 	stats1_mode_state_t* pstate = pstats1_acc->pvstate;
-	lhmsi_free(pstate->pcounts_for_value);
+	lhmsll_free(pstate->pcounts_for_value);
 	free(pstate->output_field_name);
 	free(pstate);
 	free(pstats1_acc);
@@ -173,7 +176,7 @@ stats1_acc_t* stats1_mode_alloc(char* value_field_name, char* stats1_acc_name, i
 {
 	stats1_acc_t* pstats1_acc   = mlr_malloc_or_die(sizeof(stats1_acc_t));
 	stats1_mode_state_t* pstate = mlr_malloc_or_die(sizeof(stats1_mode_state_t));
-	pstate->pcounts_for_value   = lhmsi_alloc();
+	pstate->pcounts_for_value   = lhmsll_alloc();
 	pstate->output_field_name   = mlr_paste_3_strings(value_field_name, "_", stats1_acc_name);
 
 	pstats1_acc->pvstate        = (void*)pstate;
@@ -185,6 +188,65 @@ stats1_acc_t* stats1_mode_alloc(char* value_field_name, char* stats1_acc_name, i
 	return pstats1_acc;
 }
 
+// ----------------------------------------------------------------
+typedef struct _stats1_antimode_state_t {
+	lhmsll_t* pcounts_for_value;
+	char* output_field_name;
+} stats1_antimode_state_t;
+// antimode on strings: "1" and "1.0" and "1.0000" are distinct text.
+static void stats1_antimode_singest(void* pvstate, char* val) {
+	stats1_antimode_state_t* pstate = pvstate;
+	lhmslle_t* pe = lhmsll_get_entry(pstate->pcounts_for_value, val);
+	if (pe == NULL) {
+		// lhmsll does a strdup so we needn't.
+		lhmsll_put(pstate->pcounts_for_value, mlr_strdup_or_die(val), 1, FREE_ENTRY_KEY);
+	} else {
+		pe->value++;
+	}
+}
+static void stats1_antimode_emit(void* pvstate, char* value_field_name, char* stats1_acc_name, int copy_data, lrec_t* poutrec) {
+	stats1_antimode_state_t* pstate = pvstate;
+	int min_count = 0;
+	int have_min_count = FALSE;
+	char* min_key = "";
+	for (lhmslle_t* pe = pstate->pcounts_for_value->phead; pe != NULL; pe = pe->pnext) {
+		int count = pe->value;
+		if (!have_min_count || count < min_count) {
+			min_key = pe->key;
+			min_count = count;
+			have_min_count = TRUE;
+		}
+	}
+	if (copy_data)
+		lrec_put(poutrec, mlr_strdup_or_die(pstate->output_field_name), mlr_strdup_or_die(min_key),
+			FREE_ENTRY_KEY|FREE_ENTRY_VALUE);
+	else
+		lrec_put(poutrec, pstate->output_field_name, min_key, NO_FREE);
+}
+static void stats1_antimode_free(stats1_acc_t* pstats1_acc) {
+	stats1_antimode_state_t* pstate = pstats1_acc->pvstate;
+	lhmsll_free(pstate->pcounts_for_value);
+	free(pstate->output_field_name);
+	free(pstate);
+	free(pstats1_acc);
+}
+stats1_acc_t* stats1_antimode_alloc(char* value_field_name, char* stats1_acc_name, int allow_int_float,
+	int do_interpolated_percentiles)
+{
+	stats1_acc_t* pstats1_acc   = mlr_malloc_or_die(sizeof(stats1_acc_t));
+	stats1_antimode_state_t* pstate = mlr_malloc_or_die(sizeof(stats1_antimode_state_t));
+	pstate->pcounts_for_value   = lhmsll_alloc();
+	pstate->output_field_name   = mlr_paste_3_strings(value_field_name, "_", stats1_acc_name);
+
+	pstats1_acc->pvstate        = (void*)pstate;
+	pstats1_acc->pdingest_func  = NULL;
+	pstats1_acc->pningest_func  = NULL;
+	pstats1_acc->psingest_func  = stats1_antimode_singest;
+	pstats1_acc->pemit_func     = stats1_antimode_emit;
+	pstats1_acc->pfree_func     = stats1_antimode_free;
+	return pstats1_acc;
+}
+
 // ----------------------------------------------------------------
 typedef struct _stats1_sum_state_t {
 	mv_t sum;
@@ -478,9 +540,10 @@ typedef struct _stats1_min_state_t {
 	mv_t min;
 	char* output_field_name;
 } stats1_min_state_t;
-static void stats1_min_ningest(void* pvstate, mv_t* pval) {
+static void stats1_min_singest(void* pvstate, char* sval) {
 	stats1_min_state_t* pstate = pvstate;
-	pstate->min = x_xx_min_func(&pstate->min, pval);
+	mv_t val = mv_copy_type_infer_string_or_float_or_int(sval);
+	pstate->min = x_xx_min_func(&pstate->min, &val);
 }
 static void stats1_min_emit(void* pvstate, char* value_field_name, char* stats1_acc_name, int copy_data, lrec_t* poutrec) {
 	stats1_min_state_t* pstate = pvstate;
@@ -500,6 +563,7 @@ static void stats1_min_emit(void* pvstate, char* value_field_name, char* stats1_
 }
 static void stats1_min_free(stats1_acc_t* pstats1_acc) {
 	stats1_min_state_t* pstate = pstats1_acc->pvstate;
+	mv_free(&pstate->min);
 	free(pstate->output_field_name);
 	free(pstate);
 	free(pstats1_acc);
@@ -514,8 +578,8 @@ stats1_acc_t* stats1_min_alloc(char* value_field_name, char* stats1_acc_name, in
 
 	pstats1_acc->pvstate       = (void*)pstate;
 	pstats1_acc->pdingest_func = NULL;
-	pstats1_acc->pningest_func = stats1_min_ningest;
-	pstats1_acc->psingest_func = NULL;
+	pstats1_acc->pningest_func = NULL;
+	pstats1_acc->psingest_func = stats1_min_singest;
 	pstats1_acc->pemit_func    = stats1_min_emit;
 	pstats1_acc->pfree_func    = stats1_min_free;
 	return pstats1_acc;
@@ -526,9 +590,10 @@ typedef struct _stats1_max_state_t {
 	mv_t max;
 	char* output_field_name;
 } stats1_max_state_t;
-static void stats1_max_ningest(void* pvstate, mv_t* pval) {
+static void stats1_max_singest(void* pvstate, char* sval) {
 	stats1_max_state_t* pstate = pvstate;
-	pstate->max = x_xx_max_func(&pstate->max, pval);
+	mv_t val = mv_copy_type_infer_string_or_float_or_int(sval);
+	pstate->max = x_xx_max_func(&pstate->max, &val);
 }
 static void stats1_max_emit(void* pvstate, char* value_field_name, char* stats1_acc_name, int copy_data, lrec_t* poutrec) {
 	stats1_max_state_t* pstate = pvstate;
@@ -548,6 +613,7 @@ static void stats1_max_emit(void* pvstate, char* value_field_name, char* stats1_
 }
 static void stats1_max_free(stats1_acc_t* pstats1_acc) {
 	stats1_max_state_t* pstate = pstats1_acc->pvstate;
+	mv_free(&pstate->max);
 	free(pstate->output_field_name);
 	free(pstate);
 	free(pstats1_acc);
@@ -562,8 +628,8 @@ stats1_acc_t* stats1_max_alloc(char* value_field_name, char* stats1_acc_name, in
 
 	pstats1_acc->pvstate       = (void*)pstate;
 	pstats1_acc->pdingest_func = NULL;
-	pstats1_acc->pningest_func = stats1_max_ningest;
-	pstats1_acc->psingest_func = NULL;
+	pstats1_acc->pningest_func = NULL;
+	pstats1_acc->psingest_func = stats1_max_singest;
 	pstats1_acc->pemit_func    = stats1_max_emit;
 	pstats1_acc->pfree_func    = stats1_max_free;
 	return pstats1_acc;
@@ -576,15 +642,22 @@ typedef struct _stats1_percentile_state_t {
 	int reference_count;
 	percentile_keeper_emitter_t* ppercentile_keeper_emitter;
 } stats1_percentile_state_t;
-static void stats1_percentile_ningest(void* pvstate, mv_t* pval) {
+static void stats1_percentile_singest(void* pvstate, char* sval) {
 	stats1_percentile_state_t* pstate = pvstate;
-	percentile_keeper_ingest(pstate->ppercentile_keeper, *pval);
+	mv_t val = mv_copy_type_infer_string_or_float_or_int(sval);
+	percentile_keeper_ingest(pstate->ppercentile_keeper, val);
 }
+
 static void stats1_percentile_emit(void* pvstate, char* value_field_name, char* stats1_acc_name, int copy_data, lrec_t* poutrec) {
 	stats1_percentile_state_t* pstate = pvstate;
-
 	double p;
-	(void)sscanf(stats1_acc_name, "p%lf", &p); // Assuming this was range-checked earlier on to be in [0,100].
+
+	if (stats1_acc_name[0] == 'm') { // Pre-validated to be either p{number} or median.
+		p = 50.0;
+	} else {
+		// TODO: do the sscanf once at alloc time and store the double in the state struct for a minor perf gain.
+		(void)sscanf(stats1_acc_name, "p%lf", &p); // Assuming this was range-checked earlier on to be in [0,100].
+	}
 	mv_t v = pstate->ppercentile_keeper_emitter(pstate->ppercentile_keeper, p);
 	char* s = mv_alloc_format_val(&v);
 	// For this type, one accumulator tracks many stats1_names, but a single value_field_name.
@@ -596,6 +669,7 @@ static void stats1_percentile_emit(void* pvstate, char* value_field_name, char*
 	}
 	lrec_put(poutrec, mlr_strdup_or_die(output_field_name), s, FREE_ENTRY_KEY|FREE_ENTRY_VALUE);
 }
+
 static void stats1_percentile_free(stats1_acc_t* pstats1_acc) {
 	stats1_percentile_state_t* pstate = pstats1_acc->pvstate;
 	pstate->reference_count--;
@@ -620,8 +694,8 @@ stats1_acc_t* stats1_percentile_alloc(char* value_field_name, char* stats1_acc_n
 
 	pstats1_acc->pvstate        = (void*)pstate;
 	pstats1_acc->pdingest_func  = NULL;
-	pstats1_acc->pningest_func  = stats1_percentile_ningest;
-	pstats1_acc->psingest_func  = NULL;
+	pstats1_acc->pningest_func  = NULL;
+	pstats1_acc->psingest_func  = stats1_percentile_singest;
 	pstats1_acc->pemit_func     = stats1_percentile_emit;
 	pstats1_acc->pfree_func     = stats1_percentile_free;
 	return pstats1_acc;
