@@ -1,8 +1,10 @@
 #include <stdio.h>
 #include <string.h>
 #include "lib/minunit.h"
+#include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/mlhmmv.h"
+#include "containers/mvfuncs.h"
 
 int tests_run         = 0;
 int tests_failed      = 0;
@@ -22,12 +24,12 @@ static mv_t* imv(long long intv) {
 
 // ----------------------------------------------------------------
 static char* test_no_overlap() {
-	mlhmmv_t* pmap = mlhmmv_alloc();
+	mlhmmv_root_t* pmap = mlhmmv_root_alloc();
 	int error = 0;
 
 	printf("================================================================\n");
 	printf("empty map:\n");
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
 
 	sllmv_t* pmvkeys1 = sllmv_single_with_free(imv(3));
 	mv_t value1 = mv_from_int(4LL);
@@ -35,10 +37,10 @@ static char* test_no_overlap() {
 	printf("keys1:  ");
 	sllmv_print(pmvkeys1);
 	printf("value1: %s\n", mv_alloc_format_val(&value1));
-	mlhmmv_put_terminal(pmap, pmvkeys1, &value1);
+	mlhmmv_root_put_terminal(pmap, pmvkeys1, &value1);
 	printf("map:\n");
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys1, &error), &value1));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys1, &error), &value1));
 
 	sllmv_t* pmvkeys2 = sllmv_double_with_free(smv("abcde"), imv(-6));
 	mv_t value2 = mv_from_int(7);
@@ -46,10 +48,10 @@ static char* test_no_overlap() {
 	printf("keys2:  ");
 	sllmv_print(pmvkeys2);
 	printf("value2: %s\n", mv_alloc_format_val(&value2));
-	mlhmmv_put_terminal(pmap, pmvkeys2, &value2);
+	mlhmmv_root_put_terminal(pmap, pmvkeys2, &value2);
 	printf("map:\n");
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys2, &error), &value2));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys2, &error), &value2));
 
 	sllmv_t* pmvkeys3 = sllmv_triple_with_free(imv(0), smv("fghij"), imv(0));
 	mv_t value3 = mv_from_int(0LL);
@@ -57,168 +59,168 @@ static char* test_no_overlap() {
 	printf("keys3:  ");
 	sllmv_print(pmvkeys3);
 	printf("value3: %s\n", mv_alloc_format_val(&value3));
-	mlhmmv_put_terminal(pmap, pmvkeys3, &value3);
+	mlhmmv_root_put_terminal(pmap, pmvkeys3, &value3);
 	printf("map:\n");
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys3, &error), &value3));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys3, &error), &value3));
 
 	sllmv_free(pmvkeys1);
-	mlhmmv_free(pmap);
+	mlhmmv_root_free(pmap);
 	return NULL;
 }
 
 // ----------------------------------------------------------------
 static char* test_overlap() {
-	mlhmmv_t* pmap = mlhmmv_alloc();
+	mlhmmv_root_t* pmap = mlhmmv_root_alloc();
 	int error = 0;
 
 	printf("================================================================\n");
 	sllmv_t* pmvkeys = sllmv_single_with_free(imv(3));
 	mv_t* ptermval = imv(4);
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	ptermval = imv(5);
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_double_with_free(imv(3), smv("x"));
 	ptermval = imv(6);
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	ptermval = imv(7);
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_triple_with_free(imv(3), imv(9), smv("y"));
 	ptermval = smv("z");
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_triple_with_free(imv(3), imv(9), smv("z"));
 	ptermval = smv("y");
-	mlhmmv_put_terminal(pmap, pmvkeys, ptermval);
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mlhmmv_root_put_terminal(pmap, pmvkeys, ptermval);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
-	mlhmmv_free(pmap);
+	mlhmmv_root_free(pmap);
 	return NULL;
 }
 
 // ----------------------------------------------------------------
 static char* test_resize() {
-	mlhmmv_t* pmap = mlhmmv_alloc();
+	mlhmmv_root_t* pmap = mlhmmv_root_alloc();
 	int error;
 
 	printf("================================================================\n");
 	for (int i = 0; i < 2*MLHMMV_INITIAL_ARRAY_LENGTH; i++)
-		mlhmmv_put_terminal(pmap, sllmv_single_with_free(imv(i)), imv(-i));
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
+		mlhmmv_root_put_terminal(pmap, sllmv_single_with_free(imv(i)), imv(-i));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
 	printf("\n");
 
 	for (int i = 0; i < 2*MLHMMV_INITIAL_ARRAY_LENGTH; i++)
-		mlhmmv_put_terminal(pmap, sllmv_double_with_free(smv("a"), imv(i)), imv(-i));
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
+		mlhmmv_root_put_terminal(pmap, sllmv_double_with_free(smv("a"), imv(i)), imv(-i));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
 	printf("\n");
 
 	for (int i = 0; i < 2*MLHMMV_INITIAL_ARRAY_LENGTH; i++)
-		mlhmmv_put_terminal(pmap, sllmv_triple_with_free(imv(i*100), imv(i % 4), smv("b")), smv("term"));
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
+		mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(imv(i*100), imv(i % 4), smv("b")), smv("term"));
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
 
 	sllmv_t* pmvkeys = sllmv_single_with_free(imv(2));
 	mv_t* ptermval = imv(-2);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_double_with_free(smv("a"), imv(9));
 	ptermval = imv(-9);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_double_with_free(smv("a"), imv(31));
 	ptermval = imv(-31);
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_triple_with_free(imv(0), imv(0), smv("b"));
 	ptermval = smv("term");
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_triple_with_free(imv(100), imv(1), smv("b"));
 	ptermval = smv("term");
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
 	pmvkeys = sllmv_triple_with_free(imv(1700), imv(1), smv("b"));
 	ptermval = smv("term");
-	mu_assert_lf(mv_equals_si(mlhmmv_get_terminal(pmap, pmvkeys, &error), ptermval));
+	mu_assert_lf(mv_equals_si(mlhmmv_root_look_up_and_ref_terminal(pmap, pmvkeys, &error), ptermval));
 
-	mlhmmv_free(pmap);
+	mlhmmv_root_free(pmap);
 	return NULL;
 }
 
 // ----------------------------------------------------------------
 static char* test_depth_errors() {
-	mlhmmv_t* pmap = mlhmmv_alloc();
+	mlhmmv_root_t* pmap = mlhmmv_root_alloc();
 	int error;
 
 	printf("================================================================\n");
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(3)), imv(4));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(3)), imv(4));
 
-	mu_assert_lf(NULL != mlhmmv_get_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(3)), &error));
+	mu_assert_lf(NULL != mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(3)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_NONE);
 
-	mu_assert_lf(NULL == mlhmmv_get_terminal(pmap, sllmv_triple_with_free(imv(0), imv(2), imv(3)), &error));
+	mu_assert_lf(NULL == mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_triple_with_free(imv(0), imv(2), imv(3)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_NONE);
 
-	mu_assert_lf(NULL == mlhmmv_get_terminal(pmap, sllmv_triple_with_free(imv(1), imv(0), imv(3)), &error));
+	mu_assert_lf(NULL == mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_triple_with_free(imv(1), imv(0), imv(3)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_NONE);
 
-	mu_assert_lf(NULL == mlhmmv_get_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(0)), &error));
+	mu_assert_lf(NULL == mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_triple_with_free(imv(1), imv(2), imv(0)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_NONE);
 
-	mu_assert_lf(NULL == mlhmmv_get_terminal(pmap, sllmv_quadruple_with_free(imv(1), imv(2), imv(3), imv(4)), &error));
+	mu_assert_lf(NULL == mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_quadruple_with_free(imv(1), imv(2), imv(3), imv(4)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_KEYLIST_TOO_DEEP);
 
-	mu_assert_lf(NULL == mlhmmv_get_terminal(pmap, sllmv_double_with_free(imv(1), imv(2)), &error));
+	mu_assert_lf(NULL == mlhmmv_root_look_up_and_ref_terminal(pmap, sllmv_double_with_free(imv(1), imv(2)), &error));
 	mu_assert_lf(error == MLHMMV_ERROR_KEYLIST_TOO_SHALLOW);
 
-	mlhmmv_free(pmap);
+	mlhmmv_root_free(pmap);
 	return NULL;
 }
 
 // ----------------------------------------------------------------
 static char* test_mlhmmv_to_lrecs() {
 	printf("================================================================\n");
-	mlhmmv_t* pmap = mlhmmv_alloc();
+	mlhmmv_root_t* pmap = mlhmmv_root_alloc();
 
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("x")), imv(1));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("y")), imv(2));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("x")), imv(3));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("y")), imv(4));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("x")), imv(5));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("y")), imv(6));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("x")), imv(7));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("y")), imv(8));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("u"), smv("x")), imv(9));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("u"), smv("y")), imv(10));
-	mlhmmv_put_terminal(pmap, sllmv_triple_with_free(smv("not"), smv("u"), smv("y")), imv(11));
-	mlhmmv_put_terminal(pmap, sllmv_single_with_free(smv("single")), imv(99));
-	mlhmmv_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("i")), imv(111));
-	mlhmmv_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("j")), imv(112));
-	mlhmmv_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("k")), imv(113));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("x")), imv(1));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("y")), imv(2));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("x")), imv(3));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("y")), imv(4));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("x")), imv(5));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("s"), smv("y")), imv(6));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("x")), imv(7));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("t"), smv("y")), imv(8));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("u"), smv("x")), imv(9));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("triple"), smv("u"), smv("y")), imv(10));
+	mlhmmv_root_put_terminal(pmap, sllmv_triple_with_free(smv("not"), smv("u"), smv("y")), imv(11));
+	mlhmmv_root_put_terminal(pmap, sllmv_single_with_free(smv("single")), imv(99));
+	mlhmmv_root_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("i")), imv(111));
+	mlhmmv_root_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("j")), imv(112));
+	mlhmmv_root_put_terminal(pmap, sllmv_double_with_free(smv("double"), smv("k")), imv(113));
 
 	printf("full map:\n");
-	mlhmmv_print_json_stacked(pmap, FALSE, stdout);
+	mlhmmv_root_print_json_stacked(pmap, TRUE, FALSE, "", "\n", stdout);
 	printf("\n");
 
 	sllv_t* poutrecs = NULL;
 
 	printf("----------------------------------------------------------------\n");
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("single")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("single")),
 		sllmv_alloc(), poutrecs, TRUE, ":");
 	printf("single-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -229,7 +231,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("single")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("single")),
 		sllmv_single_with_free(smv("first")), poutrecs, TRUE, ":");
 	printf("single-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -241,7 +243,7 @@ static char* test_mlhmmv_to_lrecs() {
 
 	printf("----------------------------------------------------------------\n");
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
 		sllmv_alloc(), poutrecs, TRUE, ":");
 	printf("double-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -252,7 +254,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
 		sllmv_single_with_free(smv("first")), poutrecs, TRUE, ":");
 	printf("double-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -263,7 +265,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("double")),
 		sllmv_double_with_free(smv("first"), smv("second")), poutrecs, TRUE, ":");
 	printf("double-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -275,7 +277,7 @@ static char* test_mlhmmv_to_lrecs() {
 
 	printf("----------------------------------------------------------------\n");
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
 		sllmv_alloc(), poutrecs, TRUE, ":");
 	printf("triple-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -286,7 +288,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
 		sllmv_single_with_free(smv("first")), poutrecs, TRUE, ":");
 	printf("triple-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -297,7 +299,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
 		sllmv_double_with_free(smv("first"), smv("second")), poutrecs, TRUE, ":");
 	printf("triple-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -308,7 +310,7 @@ static char* test_mlhmmv_to_lrecs() {
 	sllv_free(poutrecs);
 
 	poutrecs = sllv_alloc();
-	mlhmmv_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
+	mlhmmv_root_partial_to_lrecs(pmap, sllmv_single_with_free(smv("triple")),
 		sllmv_triple_with_free(smv("first"), smv("second"), smv("third")), poutrecs, TRUE, ":");
 	printf("triple-level outrecs (%lld):\n", poutrecs->length);
 	for (sllve_t* pe = poutrecs->phead; pe != NULL; pe = pe->pnext)
@@ -318,7 +320,7 @@ static char* test_mlhmmv_to_lrecs() {
 	printf("\n");
 	sllv_free(poutrecs);
 
-	mlhmmv_free(pmap);
+	mlhmmv_root_free(pmap);
 	return NULL;
 }
 
@@ -333,6 +335,7 @@ static char * run_all_tests() {
 }
 
 int main(int argc, char **argv) {
+	mlr_global_init(argv[0], NULL);
 	printf("TEST_MLHMMV ENTER\n");
 	char *result = run_all_tests();
 	printf("\n");
