@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <string.h>
 #include "lib/minunit.h"
+#include "lib/mlr_globals.h"
 #include "lib/mlrutil.h"
 #include "containers/slls.h"
 #include "containers/rslls.h"
@@ -8,15 +9,16 @@
 #include "lib/string_array.h"
 #include "containers/hss.h"
 #include "containers/lhmsi.h"
+#include "containers/lhmsll.h"
 #include "containers/lhmss.h"
 #include "containers/lhmsv.h"
 #include "containers/lhms2v.h"
 #include "containers/lhmslv.h"
 #include "containers/lhmsmv.h"
-#include "containers/bind_stack.h"
 #include "containers/percentile_keeper.h"
 #include "containers/top_keeper.h"
 #include "containers/dheap.h"
+#include "containers/mvfuncs.h"
 
 int tests_run         = 0;
 int tests_failed      = 0;
@@ -40,13 +42,11 @@ static char* test_slls() {
 	mu_assert_lf(plist->length == 0);
 
 	plist = slls_from_line(mlr_strdup_or_die("a"), ',', FALSE);
-	mu_assert_lf(plist->length == 1);
 
+	mu_assert_lf(plist->length == 1);
 	plist = slls_from_line(mlr_strdup_or_die("c,d,a,e,b"), ',', FALSE);
 	mu_assert_lf(plist->length == 5);
-
 	sllse_t* pe = plist->phead;
-
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "c")); pe = pe->pnext;
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "d")); pe = pe->pnext;
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "a")); pe = pe->pnext;
@@ -58,7 +58,6 @@ static char* test_slls() {
 
 	mu_assert_lf(plist->length == 5);
 	pe = plist->phead;
-
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "a")); pe = pe->pnext;
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "b")); pe = pe->pnext;
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "c")); pe = pe->pnext;
@@ -66,6 +65,32 @@ static char* test_slls() {
 	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "e")); pe = pe->pnext;
 	mu_assert_lf(pe == NULL);
 
+
+	plist = slls_from_line(mlr_strdup_or_die(","), ',', FALSE);
+	slls_print_quoted(plist);printf("\n");
+	mu_assert_lf(plist->length == 2);
+	pe = plist->phead;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "")); pe = pe->pnext;
+
+	plist = slls_from_line(mlr_strdup_or_die("a,b,c,"), ',', FALSE);
+	slls_print_quoted(plist);printf("\n");
+	mu_assert_lf(plist->length == 4);
+	pe = plist->phead;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "a")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "b")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "c")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, ""));  pe = pe->pnext;
+
+	plist = slls_from_line(mlr_strdup_or_die("a,,c,d"), ',', FALSE);
+	slls_print_quoted(plist);printf("\n");
+	mu_assert_lf(plist->length == 4);
+	pe = plist->phead;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "a")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, ""));  pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "c")); pe = pe->pnext;
+	mu_assert_lf(pe != NULL); mu_assert_lf(streq(pe->value, "d")); pe = pe->pnext;
+
 	return NULL;
 }
 
@@ -186,9 +211,9 @@ static char* test_sllv() {
 	sllv_free(pb);
 
 	pa = sllv_alloc();
-	sllv_prepend(pa, "a");
-	sllv_prepend(pa, "b");
-	sllv_prepend(pa, "c");
+	sllv_push(pa, "a");
+	sllv_push(pa, "b");
+	sllv_push(pa, "c");
 	mu_assert_lf(pa->length == 3);
 
 	pe = pa->phead;
@@ -271,11 +296,16 @@ static char* test_hss() {
 static char* test_lhmsi() {
 
 	lhmsi_t *pmap = lhmsi_alloc();
+	int val = -123;
 	mu_assert_lf(pmap->num_occupied == 0);
 	mu_assert_lf(!lhmsi_has_key(pmap, "w")); mu_assert_lf(lhmsi_get(pmap, "w") == -999);
 	mu_assert_lf(!lhmsi_has_key(pmap, "x")); mu_assert_lf(lhmsi_get(pmap, "w") == -999);
 	mu_assert_lf(!lhmsi_has_key(pmap, "y")); mu_assert_lf(lhmsi_get(pmap, "w") == -999);
 	mu_assert_lf(!lhmsi_has_key(pmap, "z")); mu_assert_lf(lhmsi_get(pmap, "w") == -999);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "x", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "y", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "z", &val) == FALSE);
 	mu_assert_lf(lhmsi_check_counts(pmap));
 
 	lhmsi_put(pmap, "x", 3, NO_FREE);
@@ -284,6 +314,10 @@ static char* test_lhmsi() {
 	mu_assert_lf( lhmsi_has_key(pmap, "x")); mu_assert_lf(lhmsi_get(pmap, "x") == 3);
 	mu_assert_lf(!lhmsi_has_key(pmap, "y")); mu_assert_lf(lhmsi_get(pmap, "y") == -999);
 	mu_assert_lf(!lhmsi_has_key(pmap, "z")); mu_assert_lf(lhmsi_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 3);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "y", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "z", &val) == FALSE);
 	mu_assert_lf(lhmsi_check_counts(pmap));
 
 	lhmsi_put(pmap, "y", 5, NO_FREE);
@@ -292,6 +326,10 @@ static char* test_lhmsi() {
 	mu_assert_lf( lhmsi_has_key(pmap, "x")); mu_assert_lf(lhmsi_get(pmap, "x") == 3);
 	mu_assert_lf( lhmsi_has_key(pmap, "y")); mu_assert_lf(lhmsi_get(pmap, "y") == 5);
 	mu_assert_lf(!lhmsi_has_key(pmap, "z")); mu_assert_lf(lhmsi_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 3);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "z", &val) == FALSE);
 	mu_assert_lf(lhmsi_check_counts(pmap));
 
 	lhmsi_put(pmap, "x", 4, NO_FREE);
@@ -300,6 +338,10 @@ static char* test_lhmsi() {
 	mu_assert_lf( lhmsi_has_key(pmap, "x")); mu_assert_lf(lhmsi_get(pmap, "x") == 4);
 	mu_assert_lf( lhmsi_has_key(pmap, "y")); mu_assert_lf(lhmsi_get(pmap, "y") == 5);
 	mu_assert_lf(!lhmsi_has_key(pmap, "z")); mu_assert_lf(lhmsi_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 4);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "z", &val) == FALSE);
 	mu_assert_lf(lhmsi_check_counts(pmap));
 
 	lhmsi_put(pmap, "z", 7, NO_FREE);
@@ -308,6 +350,10 @@ static char* test_lhmsi() {
 	mu_assert_lf( lhmsi_has_key(pmap, "x")); mu_assert_lf(lhmsi_get(pmap, "x") == 4);
 	mu_assert_lf( lhmsi_has_key(pmap, "y")); mu_assert_lf(lhmsi_get(pmap, "y") == 5);
 	mu_assert_lf(lhmsi_has_key(pmap, "z"));  mu_assert_lf(lhmsi_get(pmap, "z") == 7);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 4);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsi_test_and_get(pmap, "z", &val) == TRUE); mu_assert_lf(val == 7);
 	mu_assert_lf(lhmsi_check_counts(pmap));
 
 	lhmsi_free(pmap);
@@ -315,6 +361,75 @@ static char* test_lhmsi() {
 	return NULL;
 }
 
+// ----------------------------------------------------------------
+static char* test_lhmsll() {
+
+	lhmsll_t *pmap = lhmsll_alloc();
+	long long val = -123;
+	mu_assert_lf(pmap->num_occupied == 0);
+	mu_assert_lf(!lhmsll_has_key(pmap, "w")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf(!lhmsll_has_key(pmap, "x")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf(!lhmsll_has_key(pmap, "y")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf(!lhmsll_has_key(pmap, "z")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "x", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "y", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "z", &val) == FALSE);
+	mu_assert_lf(lhmsll_check_counts(pmap));
+
+	lhmsll_put(pmap, "x", 3, NO_FREE);
+	mu_assert_lf(pmap->num_occupied == 1);
+	mu_assert_lf(!lhmsll_has_key(pmap, "w")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf( lhmsll_has_key(pmap, "x")); mu_assert_lf(lhmsll_get(pmap, "x") == 3);
+	mu_assert_lf(!lhmsll_has_key(pmap, "y")); mu_assert_lf(lhmsll_get(pmap, "y") == -999);
+	mu_assert_lf(!lhmsll_has_key(pmap, "z")); mu_assert_lf(lhmsll_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 3);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "y", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "z", &val) == FALSE);
+	mu_assert_lf(lhmsll_check_counts(pmap));
+
+	lhmsll_put(pmap, "y", 5, NO_FREE);
+	mu_assert_lf(pmap->num_occupied == 2);
+	mu_assert_lf(!lhmsll_has_key(pmap, "w")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf( lhmsll_has_key(pmap, "x")); mu_assert_lf(lhmsll_get(pmap, "x") == 3);
+	mu_assert_lf( lhmsll_has_key(pmap, "y")); mu_assert_lf(lhmsll_get(pmap, "y") == 5);
+	mu_assert_lf(!lhmsll_has_key(pmap, "z")); mu_assert_lf(lhmsll_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 3);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "z", &val) == FALSE);
+	mu_assert_lf(lhmsll_check_counts(pmap));
+
+	lhmsll_put(pmap, "x", 4, NO_FREE);
+	mu_assert_lf(pmap->num_occupied == 2);
+	mu_assert_lf(!lhmsll_has_key(pmap, "w")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf( lhmsll_has_key(pmap, "x")); mu_assert_lf(lhmsll_get(pmap, "x") == 4);
+	mu_assert_lf( lhmsll_has_key(pmap, "y")); mu_assert_lf(lhmsll_get(pmap, "y") == 5);
+	mu_assert_lf(!lhmsll_has_key(pmap, "z")); mu_assert_lf(lhmsll_get(pmap, "z") == -999);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 4);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "z", &val) == FALSE);
+	mu_assert_lf(lhmsll_check_counts(pmap));
+
+	lhmsll_put(pmap, "z", 7, NO_FREE);
+	mu_assert_lf(pmap->num_occupied == 3);
+	mu_assert_lf(!lhmsll_has_key(pmap, "w")); mu_assert_lf(lhmsll_get(pmap, "w") == -999);
+	mu_assert_lf( lhmsll_has_key(pmap, "x")); mu_assert_lf(lhmsll_get(pmap, "x") == 4);
+	mu_assert_lf( lhmsll_has_key(pmap, "y")); mu_assert_lf(lhmsll_get(pmap, "y") == 5);
+	mu_assert_lf(lhmsll_has_key(pmap, "z"));  mu_assert_lf(lhmsll_get(pmap, "z") == 7);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "w", &val) == FALSE);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "x", &val) == TRUE); mu_assert_lf(val == 4);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "y", &val) == TRUE); mu_assert_lf(val == 5);
+	mu_assert_lf(lhmsll_test_and_get(pmap, "z", &val) == TRUE); mu_assert_lf(val == 7);
+	mu_assert_lf(lhmsll_check_counts(pmap));
+
+	lhmsll_free(pmap);
+
+	return NULL;
+}
+
 // ----------------------------------------------------------------
 static char* test_lhmss() {
 
@@ -529,8 +644,8 @@ static char* test_lhmsmv() {
 	printf("\n");
 	mu_assert_lf(pmap->num_occupied == 1);
 	mu_assert_lf(!lhmsmv_has_key(pmap, "w")); mu_assert_lf(lhmsmv_get(pmap, "w") == NULL);
-	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveq(lhmsmv_get(pmap, "x"), imv(3)));
-	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveq(lhmsmv_get(pmap, "x"), smv("3")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "x"), imv(3)));
+	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "x"), smv("3")));
 	mu_assert_lf(!lhmsmv_has_key(pmap, "y")); mu_assert_lf(lhmsmv_get(pmap, "y") == NULL);
 	mu_assert_lf(!lhmsmv_has_key(pmap, "z")); mu_assert_lf(lhmsmv_get(pmap, "z") == NULL);
 	mu_assert_lf(lhmsmv_check_counts(pmap));
@@ -540,9 +655,9 @@ static char* test_lhmsmv() {
 	printf("\n");
 	mu_assert_lf(pmap->num_occupied == 2);
 	mu_assert_lf(!lhmsmv_has_key(pmap, "w")); mu_assert_lf(lhmsmv_get(pmap, "w") == NULL);
-	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveq(lhmsmv_get(pmap, "x"), smv("3")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveq(lhmsmv_get(pmap, "y"), smv("5")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveq(lhmsmv_get(pmap, "y"), imv(5)));
+	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "x"), smv("3")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "y"), smv("5")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "y"), imv(5)));
 	mu_assert_lf(!lhmsmv_has_key(pmap, "z")); mu_assert_lf(lhmsmv_get(pmap, "z") == NULL);
 	mu_assert_lf(lhmsmv_check_counts(pmap));
 
@@ -551,8 +666,8 @@ static char* test_lhmsmv() {
 	printf("\n");
 	mu_assert_lf(pmap->num_occupied == 2);
 	mu_assert_lf(!lhmsmv_has_key(pmap, "w")); mu_assert_lf(lhmsmv_get(pmap, "w") == NULL);
-	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveq(lhmsmv_get(pmap, "x"), smv("f")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveq(lhmsmv_get(pmap, "y"), smv("5")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "x"), smv("f")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "y"), smv("5")));
 	mu_assert_lf(!lhmsmv_has_key(pmap, "z")); mu_assert_lf(lhmsmv_get(pmap, "z") == NULL);
 	mu_assert_lf(lhmsmv_check_counts(pmap));
 
@@ -561,11 +676,11 @@ static char* test_lhmsmv() {
 	printf("\n");
 	mu_assert_lf(pmap->num_occupied == 3);
 	mu_assert_lf(!lhmsmv_has_key(pmap, "w")); mu_assert_lf(lhmsmv_get(pmap, "w") == NULL);
-	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveq(lhmsmv_get(pmap, "x"), smv("f")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveq(lhmsmv_get(pmap, "y"), smv("5")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveq(lhmsmv_get(pmap, "y"), imv(5)));
-	mu_assert_lf( lhmsmv_has_key(pmap, "z")); mu_assert_lf(mveq(lhmsmv_get(pmap, "z"), smv("7")));
-	mu_assert_lf( lhmsmv_has_key(pmap, "z")); mu_assert_lf(mveq(lhmsmv_get(pmap, "z"), imv(7)));
+	mu_assert_lf( lhmsmv_has_key(pmap, "x")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "x"), smv("f")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "y"), smv("5")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "y")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "y"), imv(5)));
+	mu_assert_lf( lhmsmv_has_key(pmap, "z")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "z"), smv("7")));
+	mu_assert_lf( lhmsmv_has_key(pmap, "z")); mu_assert_lf(mveqcopy(lhmsmv_get(pmap, "z"), imv(7)));
 	mu_assert_lf(lhmsmv_check_counts(pmap));
 
 	lhmsmv_free(pmap);
@@ -573,81 +688,6 @@ static char* test_lhmsmv() {
 	return NULL;
 }
 
-// ----------------------------------------------------------------
-static char* test_bind_stack() {
-	printf("\n");
-
-	//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	bind_stack_t* pstack = bind_stack_alloc();
-	bind_stack_print(pstack);
-
-	mu_assert_lf(bind_stack_resolve(pstack, "x") == NULL);
-
-	//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	lhmsmv_t* pframe0 = lhmsmv_alloc();
-	bind_stack_push(pstack, pframe0);
-	bind_stack_print(pstack);
-
-	lhmsmv_put(pframe0, "x", smv("1"), NO_FREE);
-	lhmsmv_put(pframe0, "y", smv("2"), NO_FREE);
-	bind_stack_print(pstack);
-
-	mu_assert_lf(bind_stack_resolve(pstack, "x") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), smv("1")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), imv(1)));
-	mu_assert_lf(bind_stack_resolve(pstack, "y") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), smv("2")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), imv(2)));
-	mu_assert_lf(bind_stack_resolve(pstack, "z") == NULL);
-
-	//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	lhmsmv_t* pframe1 = lhmsmv_alloc();
-	bind_stack_push(pstack, pframe1);
-	bind_stack_print(pstack);
-
-	//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	lhmsmv_t* pframe2 = lhmsmv_alloc();
-	bind_stack_push(pstack, pframe2);
-	bind_stack_print(pstack);
-
-	mu_assert_lf(bind_stack_resolve(pstack, "x") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), smv("1")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), imv(1)));
-	mu_assert_lf(bind_stack_resolve(pstack, "y") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), smv("2")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), imv(2)));
-	mu_assert_lf(bind_stack_resolve(pstack, "z") == NULL);
-
-	lhmsmv_put(pframe2, "x", smv("three"), NO_FREE);
-	bind_stack_print(pstack);
-
-	mu_assert_lf(bind_stack_resolve(pstack, "x") != NULL);
-	mu_assert_lf(!mveq(bind_stack_resolve(pstack, "x"), smv("1")));
-	mu_assert_lf(!mveq(bind_stack_resolve(pstack, "x"), imv(1)));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), smv("three")));
-	mu_assert_lf(bind_stack_resolve(pstack, "y") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), smv("2")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), imv(2)));
-	mu_assert_lf(bind_stack_resolve(pstack, "z") == NULL);
-
-	//  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-	bind_stack_pop(pstack);
-	bind_stack_print(pstack);
-
-	mu_assert_lf(bind_stack_resolve(pstack, "x") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), smv("1")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "x"), imv(1)));
-	mu_assert_lf(bind_stack_resolve(pstack, "y") != NULL);
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), smv("2")));
-	mu_assert_lf(mveq(bind_stack_resolve(pstack, "y"), imv(2)));
-	mu_assert_lf(bind_stack_resolve(pstack, "z") == NULL);
-
-	bind_stack_free(pstack);
-
-	printf("\n");
-	return NULL;
-}
-
 // ----------------------------------------------------------------
 static char* test_percentile_keeper() {
 
@@ -829,12 +869,12 @@ static char * run_all_tests() {
 	mu_run_test(test_string_array);
 	mu_run_test(test_hss);
 	mu_run_test(test_lhmsi);
+	mu_run_test(test_lhmsll);
 	mu_run_test(test_lhmss);
 	mu_run_test(test_lhmsv);
 	mu_run_test(test_lhms2v);
 	mu_run_test(test_lhmslv);
 	mu_run_test(test_lhmsmv);
-	mu_run_test(test_bind_stack);
 	mu_run_test(test_percentile_keeper);
 	mu_run_test(test_top_keeper);
 	mu_run_test(test_dheap);
@@ -842,6 +882,8 @@ static char * run_all_tests() {
 }
 
 int main(int argc, char **argv) {
+	mlr_global_init(argv[0], NULL);
+
 	printf("TEST_MULTIPLE_CONTAINERS ENTER\n");
 	char *result = run_all_tests();
 	printf("\n");
