@@ -14,11 +14,10 @@
 # ----------------------------------------------------------------
 def main
 
-  ENV['MLR_CSV_DEFAULT_RS'] = "" # In case set in user's shell
   print make_top
 
   print make_section('NAME', [
-    "Miller is like sed, awk, cut, join, and sort for name-indexed data such as CSV and tabular JSON."
+    "Miller is like awk, sed, cut, join, and sort for name-indexed data such as CSV and tabular JSON."
   ])
 
   print make_section('SYNOPSIS', [
@@ -64,6 +63,9 @@ output separator to the given value."""
 	print make_subsection('I/O FORMATTING', [])
 	print make_code_block(`mlr --usage-data-format-options`)
 
+	print make_subsection('FORMAT-CONVERSION KEYSTROKE-SAVERS', [])
+	print make_code_block(`mlr --usage-format-conversion-keystroke-saver-options`)
+
 	print make_subsection('COMPRESSED I/O', [])
 	print make_code_block(`mlr --usage-compressed-data-options`)
 
@@ -108,7 +110,7 @@ output separator to the given value."""
   end
 
   keywords = `mlr --list-all-keywords-raw`
-  print make_section('KEYWORDS FOR PUT', [
+  print make_section('KEYWORDS FOR PUT AND FILTER', [
     ""
   ])
   keywords = keywords.strip.split("\n").uniq
@@ -123,7 +125,7 @@ output separator to the given value."""
     "This manual page has been composed from Miller's help output by Eric MSP Veith <eveith@veith-m.de>."
   ])
   print make_section('SEE ALSO', [
-    "sed(1), awk(1), cut(1), join(1), sort(1), RFC 4180: Common Format and MIME Type for " +
+    "awk(1), sed(1), cut(1), join(1), sort(1), RFC 4180: Common Format and MIME Type for " +
     "Comma-Separated Values (CSV) Files, the miller website http://johnkerl.org/miller/doc"
   ])
 end
